#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <net/if.h>
#include <string.h>
#include <sys/file.h>
#include <sys/time.h>
#include <ctype.h>
#include "otto_measure.h"

static void usage ()
{
	fprintf (stderr, "usage: atmstat [-I interface] [-z] [-n] [-i interval] vci/all ...\n");
	exit (2);
}

static void null (int sig)
{
}

static void usleep (unsigned long us)
{
	struct itimerval it;
	it.it_value.tv_sec = us / 1000000;
	it.it_value.tv_usec = us % 1000000;
	it.it_interval.tv_sec = 0;
	it.it_interval.tv_usec = 0;
	signal (SIGALRM, null);
	setitimer (ITIMER_REAL, &it, (struct itimerval *) 0);
	pause ();
}

main (int argc, char *argv[])
{
	int argn;
	unsigned long interval = 0;
	char interface[1024];
	char label[11];
	struct otto_measure m;
	struct otto_counts *inc;
	int count = 0;
	int fd;
	int rc = 0;
	int nonzero = 0;
	int n;

	strcpy (interface, "/dev/otto0");
	strcpy (label, "otto0 VC");
	m.cmd = OTTO_C_MK (OTTO_C_GET);

	for (argn = 1; argn < argc && argv[argn][0] == '-'; argn++) {
		char *flags;
		for (flags = &argv[argn][1]; *flags != '\0'; flags++) {
			switch (*flags) {
			case 'z':
				m.cmd = OTTO_C_MK (OTTO_C_ZERO);
				break;
			case 'n':
				nonzero = 1;
				break;
			case 'i':
				if (++argn < argc) {
					char *arg = argv[argn];
					interval = 0;
					while (isdigit (*arg)) {
						interval *= 10;
						interval += *arg - '0';
						arg++;
					}
					if (*arg == 'u') {
						/* nothing */
					} else if (*arg == 'm') {
						interval *= 1000;
					} else {
						interval *= 1000000;
					}
				} else {
					usage ();
				}
				break;
			case 'I':
				if (++argn < argc) {
					strcpy (interface, "/dev/");
					strcat (interface, argv[argn]);
					sprintf (label, "%.7s VC", argv[argn]);
				} else {
					usage ();
				}
				break;
			default:  usage ();
			}
		}
	}

	if ((fd = open (interface, O_RDONLY, 0)) < 0) {
		perror (interface);
		exit (2);
	}

	if (argn >= argc) {
		static char *staticargv[] = { "all" };
		argv = staticargv;
		argn = 0;
		argc = 1;
	}

	n = nonzero? 2048: (argc-argn);
	inc = (struct otto_counts *)
		malloc (sizeof (struct otto_counts) * n);
	if (inc == (struct otto_counts *) 0) {
		perror ("malloc");
		exit (2);
	}

	do {
		int i;
		if (count == 0) {
			printf ("%-10s %12s %12s %12s %12s\n", label,
			    "rcv pkts", "rcv cells", "xmt pkts", "xmt cells");
			bzero (inc, sizeof (struct otto_counts) * n);
		}

		for (i = 0; i != n; i++) {
			if (nonzero) {
				m.vc = i;
			} else {
				if (strcmp (argv[i+argn], "all") == 0) {
					m.vc = 0;
				} else {
					m.vc = atoi (argv[i+argn]);
				}
			}
			if (ioctl (fd, OTTO_MEASURE, (char *) &m) == -1) {
				char err[100];
				fflush (stdout);
				if (m.vc == 0) {
					fprintf (stderr, "%-10s ", "all");
				} else {
					fprintf (stderr, "%-10u ", m.vc);
				}
				perror ("OTTO_MEASURE");
				rc = 1;
			} else {
				unsigned a = 
				  m.uu.counts.perevent[OTTO_EVENT_RX].pkts -
					inc[i].perevent[OTTO_EVENT_RX].pkts;
				unsigned b = 
				  m.uu.counts.perevent[OTTO_EVENT_RX].cells -
					inc[i].perevent[OTTO_EVENT_RX].cells;
				unsigned c = 
				  m.uu.counts.perevent[OTTO_EVENT_TX].pkts -
					inc[i].perevent[OTTO_EVENT_TX].pkts;
				unsigned d = 
				  m.uu.counts.perevent[OTTO_EVENT_TX].cells -
					inc[i].perevent[OTTO_EVENT_TX].cells;
				if (!nonzero ||
				    a != 0 || b != 0 || c != 0 || d != 0) {
					if (m.vc == 0) {
						printf ("%-10s ", "all");
					} else {
						printf ("%-10u ", m.vc);
					}
					printf ("%12u %12u %12u %12u\n",
						a, b, c, d);
					if (m.cmd == OTTO_C_MK (OTTO_C_GET)) {
						inc[i] = m.uu.counts;
					}
				}
			}
		}
		count = (count + 1) % 22;
		if (interval > 0) {
			fflush (stdout);
			usleep (interval);
		}
	} while (interval > 0);
	(void) close (fd);
	exit (rc);
}
