/* Copyright 1993, 1994 Digital Equipment Corporation.               */
/* Distributed only by permission.                             */
/* Last modified on Thu Sep 15 14:04:59 1994 by burrows
        modified on Tue Jun 28 10:49:52 CDT 1994 by evans (jbe)
        modified on Tue Apr 19, 1994 by Ted Ross
 */


#include "otto.h"
#if (NOTTO > 0) || defined(BINARY)

#include "if_otto_data.c"

volatile otto_uword32 *ottoclockloc;

extern	struct protosw *iftype_to_proto(), *iffamily_to_proto();
extern unsigned min ();
extern	struct timeval	time;
int	ottoattach(), ottointr(), ottoprobe(), otto_ifstart(),
        ottooutput();

int	ottoinit(), ottoifioctl(), ottowatch(), ottoioctl();
void otto_simplesendpkt ();

#if (NPFWD > 0)
ottobridging(sc, isbridging)
struct otto_softc *sc;
int isbridging;
{
	sc->otto_isbridging = isbridging;
}
#endif

extern int cpu;			/* identifiers are in cpuconf.h */

STD_TYPE ottostd[] = { 0 };
struct  DRIVER_STRUCT ottodriver =
#ifdef __osf__
	{ ottoprobe, 0, ottoattach, 0, 0, ottostd, 0, 0, "otto", ottoinfo };
#else 
	{ ottoprobe, 0, ottoattach, 0,    ottostd,       "otto", ottoinfo };
#endif

int nootto = 0;		/* if set, driver is disabled */
int ottodebug = 0;
int forceotto;	/* UNUSED */

static void initring ();
static void ottorestart ();
static void ottolockedrestart ();
static struct mbuf *ottoread ();
static void queuercvbuffers ();
static int replenishrcvbuffer ();
static void freequeuedmbufs ();
static struct mbuf *copypacket ();
static void initsoftc ();
#define ottodprintf(lev, x)	do { if (ottodebug >= (lev)) { cprintf x;} } while (0)

#define limitprintf(l, x) do {					\
			static long lasttime;			\
			static int limitcount;			\
			limitcount++;				\
			if (time.tv_sec > lasttime+(l)) {	\
				cprintf ("otto: count=%d ", limitcount);\
				cprintf x;			\
				lasttime = time.tv_sec;		\
			}					\
		} while (0)

/*---------------------------------------------*/

#define OTTO_PKTROUNDUP 48	/* round up packets to a cell size */
#define OTTO_RCVPKTS	(OTTO_MAXPKTSPERRING-1)	/* no. rcv pkts outstanding */
#define OTTO_CLPERRCVPKT	((10*1024+M_CLUSTERSZ-1)/M_CLUSTERSZ)
	/* mbuf clusters per receive packet  - allow for 10 KByte packets */

static fc_locktmp_t otto_fc_lock (struct otto_softc *sc);
static void otto_fc_unlock (struct otto_softc *sc, fc_locktmp_t s);
static unsigned otto_fc_mstowait (struct otto_softc *sc, unsigned first, 
	unsigned last, unsigned firstcall, struct fc_ackstate *as);
static void otto_fc_stop (struct otto_softc *sc, unsigned first, unsigned last);
static void otto_fc_start (struct otto_softc *sc, unsigned first, unsigned last,
	unsigned adjustcredits);
static void otto_fc_send (struct otto_softc *sc, unsigned char *p, unsigned l);
static void otto_fc_zaplink (struct otto_softc *sc);

/* there is one list shared by all OTTOs */
struct mbuf *otto_listsimplesend;	/* protected by otto_locksimplesend */
OTTO_SMP_LOCKTYPE otto_locksimplesend;


/*
 * Probe the OTTO to see if it's there 
 */
ottoprobe(reg,ui)
	caddr_t reg;
	struct	CONTROLLER_STRUCT *ui;
{
	int unit = ui->UNIT_NUM;
	struct otto_softc *sc = &otto_softc[unit];
	int i;
	caddr_t addr, paddr, opaddr;
	struct otto_xilinxdesc xilinxcode;
	struct fc_controller_state fccs;
	char devromname[TC_ROMNAMLEN + 1];
	int ispotto;
	extern u_char otto_xilinxdata[];
	extern int otto_xilinxdatalen;
	extern int otto_xilinxchecksum;

	if (unit == 0) {
		OTTO_SMP_LOCKINIT (&otto_locksimplesend);
		otto_listsimplesend = (struct mbuf *) 0;
	}

	/* test whether driver disabled from boot command line */
	otto_nootto ();
        if (nootto) {
                cprintf ("otto%d: OTTO driver disabled\n", unit);
                return (0);
        }

	/* allocate space for the report ring */
#ifdef NO_CONTIGUOUS_MALLOC
	/* OSF allocator can't handle requests for contiguous memory.
	   At least, we haven't found a way. */
	addr = sc->data.reportBuf;
#else
	KM_ALLOC (addr, caddr_t, OTTO_REP_RING_LEN_BYTES * 2,
	    KM_DEVBUF, KM_NOW_CL_CO_CA|KM_NOCACHE);
#endif

	if (addr == 0) {
		cprintf ("otto%d: probe failed: couldn't allocate memory for report ring\n",
			unit);
		return (0);
	}

	/* force natural alignment of physical address */
	opaddr = paddr = (caddr_t) OTTO_SVTOPHY (addr);
	paddr += OTTO_REP_RING_LEN_BYTES;
	paddr = (caddr_t)
		(((unsigned long) paddr) & ~(OTTO_REP_RING_LEN_BYTES - 1));

	/* check to make sure memory is indeed contiguous */
	/* this should never fail, since we use KM_NOW_CL_CO_CA above */
	addr += paddr - opaddr;
	for (i=0; i < OTTO_REP_RING_LEN_BYTES; i+=512) {
		if (OTTO_SVTOPHY (addr) + i != OTTO_SVTOPHY (addr+i)) {
			cprintf ("otto%d: probe failed: report ring non-contiguous\n", unit);
			return (0);
		}
	}

	tc_addr_to_name (reg, devromname);	/* find out type of device */
	ispotto = (bcmp (devromname, "DGLTA-FA", 8) == 0);

	addr = (caddr_t) PHYS_TO_K1 (paddr);	
		/* report ring is uncached on MIPS */

	/* fill in addresses of main sections of OTTO slot */
#ifdef __alpha
	reg = (caddr_t) (PHYS_TO_KSEG(DENSE(KSEG_TO_PHYS(reg))));
#else
	reg = (caddr_t) PHYS_TO_K1 ((u_long) reg);
#endif
	bzero ((char *)&sc->data, sizeof (sc->data));

	xilinxcode.data = otto_xilinxdata;
	xilinxcode.stride = 1;
	xilinxcode.checksum = otto_xilinxchecksum;
	xilinxcode.len = otto_xilinxdatalen;
#ifdef DEC_3000_300
	if (!ispotto) {
		/* Machines with slow TURBOchannels have slightly 
		   different Xilinx code.    The loop below applies
		   the patches.
		   This code isn't needed on product OTTOs, which 
		   have an extra crystal instead of using the TC clock  */
		extern u_char otto_celldiff[];
		extern int otto_cellxilinxstart;
		extern int otto_slowxilinxchecksum;
		if (cpu == DEC_3000_300 &&
		    xilinxcode.checksum != otto_slowxilinxchecksum) {
			unsigned bit = otto_cellxilinxstart * 8 - 1;
			cprintf ("using pelican Xilinx code\n");
			i = 0;
			while (otto_celldiff[i] != 0) {
				bit += otto_celldiff[i++];
				if (otto_celldiff[i] == 0) {
					i++;
				} else {
					unsigned byte = bit / 8;
					xilinxcode.data[byte] ^= (1 << (bit&7));
				}
			}
			xilinxcode.checksum = otto_slowxilinxchecksum;
		}
	}
#endif

	if (otto_initialize (unit,  reg, &sc->data, &xilinxcode,
			OTTO_TXCELLS, OTTO_NOACKCELLS, addr) != 0) {
		cprintf ("otto%d: probe failed\n", unit);
		return (0);
	}
	otto_map_init (&sc->data.vcmap);
	otto_map_init (&sc->data.ipvcmap);
	otto_map_init (&sc->data.portvcmap);

	OTTO_SMP_LOCKINIT (&od->schedlock);
	OTTO_SMP_LOCKINIT (&od->execlock);

	sc->data.schedlen = OTTO_SCHEDSIZE / 2;
	sc->data.schedfree = sc->data.schedlen;

	if (unit == 0) {
		ottoclockloc = &sc->data.dma->ticks;
	}

	/* initialize the lock */
	OTTO_SMP_LOCKINIT(&sc->data.lk_otto_softc);

	fccs.arg = sc;
	fccs.lock = otto_fc_lock;
	fccs.unlock = otto_fc_unlock;
	fccs.mstowait = otto_fc_mstowait;
	fccs.stopcircuits = otto_fc_stop;
	fccs.startcircuits = otto_fc_start;
	fccs.send = otto_fc_send;
	fccs.zaplink = otto_fc_zaplink;
	sc->data.fcs = fc_init (&fccs);   /* Init flow control resync */

	sc->data.vcs[0].flags = OTTO_VC_FIXED;
	sc->data.vcs[OTTO_FS_VCI].flags = OTTO_VC_FIXED;
	sc->data.vcs[OTTO_FS_VCI].encap = OTTO_EN_RAW;
	sc->data.vcs[OTTO_FS_VCI].etype = 0;
	sc->data.vcs[OTTO_FS_VCI].q = OTTO_IFQ_SPECIAL;

	sc->data.xmtqueues.totallen = 0;/* no packets in incoming xmt queues */
	sc->data.xmtqueues.maxtotallen = IFQ_MAXLEN;
	sc->data.xmtqueues.ready = (struct otto_xmtq *) 0;

	for (i = 0; i != OTTO_MAXIFQS; i++) {
		sc->data.xmtqueues.per_q[i].maxpktswaiting = 1;
		sc->data.xmtqueues.per_q[i].pifq =
			&sc->data.xmtqueues.per_q[i].xifq;
		sc->data.xmtqueues.per_q[i].xifq.ifq_maxlen = 20;
		IFQ_LOCKINIT (&sc->data.xmtqueues.per_q[i].xifq);
	}
	sc->data.xmtqueues.per_q[OTTO_IFQ_NORMAL].count = 100000;
	sc->data.xmtqueues.per_q[OTTO_IFQ_SPECIAL].count = 100000;
	sc->data.xmtqueues.per_q[OTTO_IFQ_NORMAL].pifq = &sc->is_if.if_snd;
	sc->is_if.if_snd.ifq_maxlen = sc->data.xmtqueues.maxtotallen/2;
	sc->data.xmtqueues.per_q[OTTO_IFQ_NORMAL].maxpktswaiting = 4;

	sc->data.ispotto = ispotto;

	return (sizeof (struct otto_softc));
}

/* initialize softc after a device soft reset */
/* should be called with softc lock held */
static void initsoftc (sc)
struct otto_softc *sc;
{
	unsigned ring;
	int i;

	sc->data.xmtqueues.pktswaiting = 0;	/* no xmt packets waiting in controllr*/
	sc->data.xmtqueues.byteswaiting = 0;	/* no xmt bytes waiting in controller */
	/* no packets or bytes waiting in the controller from any queue */
	for (i = 0; i != OTTO_MAXIFQS; i++) {
		sc->data.xmtqueues.per_q[i].pktswaiting = 0;
	}

	ring = otto_allocring (&sc->data);	/* allocate default ring */
	sc->data.ram[OTTO_DEFAULTRING] =	/* tell OTTO default ring */
		OTTO_MAKE_DEFAULTRING (sc->data.rings[ring].desc);
	initring (sc, ring);
	sc->data.defaultring = ring;
	queuercvbuffers (sc, ring, OTTO_RCVPKTS, (struct mbuf *) 0, 0);

	sc->data.schedhalf = 0;
	otto_newsched (&sc->data);
	for (i = 1; i != OTTO_MAXVC; i++) {
		struct otto_vcdata *vcdata = &sc->data.vcs[i];

		otto_setrxvc (&sc->data, i,
			vcdata->flags & OTTO_VC_IGNORE, 1,
			0, sc->data.rings[ring].desc);
		vcdata->rxring = ring;
		otto_settxvc (&sc->data, i, vcdata->flags & OTTO_VC_SCHED,
			vcdata->flags & OTTO_VC_FC, -1, -1);
	}
	otto_fakesnmp_updown (sc, 0);
	sc->data.otto_watchdog.notgettingcells = 1;
	otto_setled (&sc->data, sc->data.linkinuse, 0);
}

/* shutdown the OTTO */
/* called with softc lock held */
static int ottoreset (unit, unused)
int unit;
int unused;		/* more stupidity from OSF/1 V3.0 */
{
	struct otto_softc *sc = &otto_softc[unit];
	struct ifnet *ifp = &sc->is_if;

	ifp->if_flags &= ~(IFF_RUNNING); /*not running*/
	otto_setreset (&sc->data);
	freequeuedmbufs (sc);
	printf("otto%d: otto shut down\n", unit);
	return (0);
}


/* reset and restart the OTTO */
/* softc lock not held on entry */
static void ottorestart (unit)
int unit;
{
	struct otto_softc *sc = &otto_softc[unit];
	OTTO_SMP_LOCKTMP s;

	OTTO_SMP_LOCK (&sc->lk_otto_softc, s);
	ottolockedrestart (sc);
	OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);
}

/* reset and restart the OTTO */
/* softc lock held on entry */
static void ottolockedrestart (sc)
struct otto_softc *sc;
{
	struct ifnet *ifp = &sc->is_if;
	int running;

	cprintf ("otto%d: restart\n", ifp->if_unit);

	running = (ifp->if_flags & IFF_RUNNING);
	ifp->if_flags &= ~(IFF_RUNNING);

	otto_setreset (&sc->data);
	freequeuedmbufs (sc);

	if (running) {
		/* reinitialize the device and the free lists for its resources */
		if (otto_softreset (&sc->data, OTTO_TXCELLS, OTTO_NOACKCELLS) != 0) {
			panic ("otto_softreset failed");
		}
		initsoftc (sc);

		/* Mark interface up and running; start output on device.  */
		sc->is_if.if_flags |= (IFF_UP | IFF_RUNNING);
		sc->is_if.if_flags &= ~IFF_OACTIVE; 

		OTTO_IFSTART (&sc->is_if);		/* queue output packets */
	}
}



ottointr (unit)
int unit;
{
	struct otto_softc *sc = &otto_softc[unit];
	struct ifnet *ifp = &sc->is_if;
	volatile struct otto_rep *reportbase = sc->data.reportbase;
	OTTO_SMP_LOCKTMP s;
	otto_uword32 reporttag;
	int nextreport;
	int link = 0;		/*XXX*/

	OTTO_SMP_LOCK (&sc->lk_otto_softc, s);

	if (sc->data.deferred) {
		sc->data.deferred = 0;
		otto_unlockpioonly (&sc->data);
	}

	reporttag = sc->data.reporttag;
	nextreport = sc->data.nextreport;

	/* process each report */
	do {
		otto_uword32 seqreport;
		/* pick up the DMA report word (point 1) */
		otto_uword32 dmareport = reportbase[nextreport].dmareport;		/*1*/
		unsigned bytes;
		unsigned vc;

		if ((dmareport & OTTO_REP_DMA_TAGS) != reporttag) {
			/* turn off the interrupt condition */
			wbflush ();
#ifdef __alpha
			/* On Alpha, we acknowlegde the interrupt with a
			   read to avoid write-buffer flush problems. */
			{ volatile int readback = sc->data.ram_intack[0]; }
#else
			sc->data.ram_intack[0] = 0;	/*(point 2)*/
#endif
			wbflush ();

			/* save time when we last handled a report.
			   This allows us to fake transmit interrupts
			   in ottooutput(). */
			sc->data.otto_watchdog.lastinterrupt =
				time.tv_usec;

			/* Check the DMA report word again to avoid
			   losing interrupt for any packet delivered
			   between points *1* and *2* above */
			dmareport = reportbase[nextreport].dmareport;
			if ((dmareport & OTTO_REP_DMA_TAGS) != reporttag) {
				break;
			}
		}

		seqreport = reportbase[nextreport].seqreport;
		bytes = OTTO_REP_DMA_BYTECOUNT (dmareport);
		vc = OTTO_REP_SEQ_VCI (seqreport);

		if ((OTTO_REP_SEQ_ISXMT & seqreport) == 0) {
			/* receive */
			unsigned pktdesc;
			struct otto_rxpkt *rxpkt;
			struct mbuf *m;
			struct otto_ring *ring =
				&sc->data.rings[sc->data.vcs[vc].rxring];
			unsigned tmp;
			unsigned rxfragdescindex;
			volatile struct otto_descriptor *rcvring = 
				&sc->data.descriptors[ring->desc+1];
			unsigned nextfree = ring->nextfree;
			unsigned lastused = ring->lastused+1;
			unsigned tag = ring->tag;
			unsigned rxpktindex;

			sc->is_if.if_ipackets++;

			pktdesc = OTTO_REP_SEQ_PKT (seqreport);
			if (pktdesc == OTTO_EMPTYPKT) {
				ottodprintf (3, ("ottointr: rcv %d bytes into empty buffer on vc %d\n",
					bytes,vc));
				goto next_report;
			}

			tmp = (pktdesc - OTTO_RINGSSTART) /
				(OTTO_MAXPKTSPERRING+1);

			ottodprintf (3, (
				"otto: rcv into entry %d of ring %d vc=%d savedring=%d pkt=%x lastused=%d\n",
				tmp, ring - sc->data.rings, vc,
				sc->data.vcs[vc].rxring,
				OTTO_REP_SEQ_PKT (seqreport),
				ring->lastused));

			if (&sc->data.rings[tmp] != ring) {
				limitprintf (5,
				  ("ottointr: ring number isn't right %u %ld\n",
					tmp, (long) (sc->data.rings-ring)));
				ring = &sc->data.rings[tmp];
				rcvring = &sc->data.descriptors[ring->desc+1];
				nextfree = ring->nextfree;
				lastused = ring->lastused+1;
				tag = ring->tag;
			}
			tmp = pktdesc - ring->desc - 1;

			rxpktindex = ring->pktindex[tmp];
			rxpkt = &sc->data.rxpkts[rxpktindex];
			m = rxpkt->m;
			rxfragdescindex = rxpkt->desc;
			tag = OTTO_MAKE_RXPKT (0, 0, tag, rxfragdescindex);

			ottodprintf (6, ("vc:%x:%x  ring:%x:%x pkt:%x:%x frag:%x:%x %x\n",
			    &sc->data.ram[OTTO_VCPOS+OTTO_MAXVC+vc],
			    *(otto_uword32 *)&sc->data.ram[OTTO_VCPOS+OTTO_MAXVC+vc],
			    &sc->data.descriptors[ring->desc],
			    *(otto_uword32 *)&sc->data.descriptors[ring->desc],
			    &rcvring[ring->lastused],
			    *(otto_uword32 *)&rcvring[ring->lastused],
			    &sc->data.descriptors[rxfragdescindex],
			    *(otto_uword32 *)&sc->data.descriptors[rxfragdescindex],
			    ((otto_uword32 *)&sc->data.descriptors[rxfragdescindex])[1]
			    ));

			if (m == (struct mbuf *) 0) {
				int kk;
				ottodprintf (0, (
				    "pkt_in_ring=%d ring: tag=%x desc=%x next=%d nextfree=%d lastused=%d missed=%d\n",
				    tmp, ring->tag, ring->desc,
				    ring->next, ring->nextfree,
				    ring->lastused, ring->missed));
				for (kk = 0; kk != OTTO_MAXPKTSPERRING; kk++) {
					ottodprintf (0, ("%x ", ring->pktindex[kk]));
				}
				ottodprintf (0, ("\n"));
				panic ("ottointr: null receive mbuf");
			}

			if (lastused == OTTO_MAXPKTSPERRING) {
				lastused = 0;
			}
			ring->lastused = lastused;

			/* packet must be at least a cell, less than 
			   64Kbytes, 0mod4 bytes long, and no error bits
			   should be set */
#if 0
			if (bytes >= OTTO_PKTROUNDUP &&
			    (bytes & 3) == 0 &&
			    bytes < 0x10000 &&
			    (dmareport & (OTTO_REP_DMA_CRCERROR|
			                  OTTO_REP_DMA_DMAERR)) == 0) {
#endif
			/* XXX - temporarily ignore bad CRCs from switch on
			   credit VC (0xcc) */
			if (bytes >= OTTO_PKTROUNDUP &&
			    (bytes & 3) == 0 &&
			    bytes < 0x10000 &&
			    (((dmareport & OTTO_REP_DMA_CRCERROR) == 0) ||
			     (vc == 0xcc)) &&
			    (dmareport & OTTO_REP_DMA_DMAERR) == 0) {
				int used;

				sc->data.otto_watchdog.gotrcv =
					sc->data.otto_watchdog.time;

				m = ottoread (sc, m, bytes, vc, link, &used);
				/* m is now the tail of the mbuf chain.
				   The first "used" elements of the 
				   original chain have been removed.

				   The next line allocates new mbufs to
				   take the place of those removed. */
			        if (!replenishrcvbuffer (sc, rxpkt, used, m)) {
					/* if we can't get enough mbufs,
					   defer interrupts for a while */
					if (ring->missed == 0) {
						ring->next =
						     sc->data.ringmissedlist;
						sc->data.ringmissedlist = 
						    ring - sc->data.rings;
					}
					ring->missed++;
					sc->data.deferred = 1;
					otto_defer (&sc->data);
					goto next_report;
			        }
			} else {
				/* packet had an error */

				sc->is_if.if_ierrors++;
                                if (sc->is_ctrblk.est_recvfail != 0xffff) {
                                        sc->is_ctrblk.est_recvfail++;
                                }
				OTTO_STATS_INC_L (&sc->data, rcv_error, link);
				if (dmareport & OTTO_REP_DMA_DMAERR) {
					/* this error will probably never be
					   seen.  If there was a DMA error,
					   how did the DMA engine DMA the report
					   tell us?   We shutdown the device,
					   and get the watchdog to restart. */
				        otto_setreset (&sc->data);
				        sc->data.otto_watchdog.forcereset = 1;
                                        limitprintf (5,
                                            ("ottointr: rcv dma error on %d\n",
                                            link));
                                        OTTO_STATS_INC_L (&sc->data,
                                        	rcv_dma_error, link);
					OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);
					return;
				}
				if (dmareport & OTTO_REP_DMA_CRCERROR) {
                                        sc->is_ctrblk.est_recvfail_bm |= (1<<0);
                                        sc->is_ctrblk.est_multiple++;
                                        OTTO_STATS_INC_L (&sc->data,
                                        	rcv_crc_error, link);
					ottodprintf (2, ("crc error\n"));
				}
				if ((bytes % 4) != 0) {
					cprintf ("ottointr: byte not 0mod4\n");
					OTTO_STATS_INC_L (&sc->data, 
						rcv_len_err, link);
				}
				if (bytes > 0x10000) {
					cprintf ("ottointr: rcv pkt too long\n");
					OTTO_STATS_INC_L (&sc->data, 
						rcv_pkt_too_large, link);
				}
				if (bytes < OTTO_PKTROUNDUP) {
					cprintf ("ottointr: rcv pkt too short bytes=%d rep=%x\n",
						bytes, dmareport);
					OTTO_STATS_INC_L (&sc->data,
						rcv_pkt_too_short, link);
				}
			}
			wbflush ();
			OTTO_WRITE_LOW (&rcvring[nextfree].word0, tag, 0xffff);
			wbflush ();
			sc->data.otto_watchdog.queuedrcvbuf =
				sc->data.otto_watchdog.time;
			ring->pktindex[nextfree] = rxpktindex;
			nextfree++;
			if (nextfree == OTTO_MAXPKTSPERRING) {
				nextfree = 0;
				ring->tag = OTTO_TAG_FLIP (ring->tag);
			}
			ring->nextfree = nextfree;

			/* try to get back any packet we were not able to
			   allocate before */
			if (ring->missed != 0) {
				queuercvbuffers (sc,
				    (unsigned) (ring - sc->data.rings), 0,
				    (struct mbuf *) 0, 0);
			}
		} else {
			/* transmit */
			unsigned *txvcpkt = &sc->data.vcs[vc].txpkts;
			unsigned lastpkt = *txvcpkt;
			struct otto_txpkt *lasttxpkt= &sc->data.txpkts[lastpkt];
			unsigned firstpkt = lasttxpkt->next;
			struct otto_txpkt *firsttxpkt =
				&sc->data.txpkts[firstpkt];

			if (lastpkt == 0) {
				panic ("ottointr: no TX pkt");
			}

			if (seqreport & OTTO_REP_SEQ_SENT) {
				/* packet has gone down the wire */
				struct otto_xmtq *xmtq =
				  &sc->data.xmtqueues.per_q[firsttxpkt->q];

				ottodprintf (3, ("otto: pkt sent\n"));

				if (firsttxpkt->m != (struct mbuf *) 0) {
					panic ("otto: tx didn't free mbuf");
				}
				sc->data.xmtqueues.byteswaiting -=
					firsttxpkt->len;
				sc->data.xmtqueues.pktswaiting--;
				if (sc->data.xmtqueues.pktswaiting == 0) {
					sc->data.xmtqueues.byteswaiting = 0;
					ifp->if_flags &= ~IFF_OACTIVE;
				}
				xmtq->pktswaiting--;
				if (xmtq->next==(struct otto_xmtq *)0 &&
				    xmtq->pifq->ifq_head &&
				    xmtq->pktswaiting < xmtq->maxpktswaiting) {
					/* put this queue on the ready list */
					struct otto_xmtq *lastxmtq =
						sc->data.xmtqueues.ready;
					if (lastxmtq == (struct otto_xmtq *)0) {
						xmtq->next = xmtq;
					} else {
						xmtq->next = lastxmtq->next;
						lastxmtq->next = xmtq;
					}
					sc->data.xmtqueues.ready = xmtq;
				}
				sc->is_if.if_opackets++;
				sc->data.vcs[vc].flags |= OTTO_VC_XMTDONE;
				sc->data.otto_watchdog.gotxmt =
					sc->data.otto_watchdog.time;
				/* remove first packet from list of outstanding
				   transmit packets on this VC */
				if (firstpkt == lastpkt) {
					/* there was only one pkt outstanding */
					*txvcpkt = 0;
				} else {
					lasttxpkt->next = firsttxpkt->next;
				}
				otto_freetxpkt (&sc->data, firstpkt);
			} else {
				/* packet is now in the adapter */
				struct otto_txpkt *curtxpkt = firsttxpkt;

				ottodprintf (3, ("otto: pkt dmaed\n"));

				if (dmareport & OTTO_REP_DMA_DMAERR) {
					/* a DMA error here means serious
					   trouble.  Shutdown the device and
					   get the watchdog to restart. */
				        otto_setreset (&sc->data);
				        sc->data.otto_watchdog.forcereset = 1;
                                        limitprintf (5,
                                            ("ottointr: xmt dma error on %d\n",
                                            link));
                                        OTTO_STATS_INC_L (&sc->data,
                                        	xmt_dma_error, link);
					OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);
					return;
				}
				/* find first packet not yet reported DMAed
				   that is on VC's transmit queue */
				while (curtxpkt->m == (struct mbuf *) 0) {
					if (curtxpkt == lasttxpkt) {
						ottodprintf (0, ("seqreport=%x dmareport=%x vc=%d\n",
						    seqreport, dmareport, vc));
						/* there must be such a packet*/
						panic ("otto: TX queue broken");
					}
					curtxpkt = &sc->data.txpkts[
						curtxpkt->next];
				}

				OTTOMFREEM (curtxpkt->m);
				curtxpkt->m = (struct mbuf *) 0;
			}
		}

next_report:
		/* step on to the next report */
		nextreport++;
		if (nextreport == OTTO_REP_RING_LEN_REPS) {
			nextreport = 0;
			reporttag = OTTO_REP_TAGFLIP (reporttag);
		}
	} while (!sc->data.deferred);
	sc->data.nextreport = nextreport;
	sc->data.reporttag = reporttag;

	if (sc->data.xmtqueues.pktswaiting < 0) {
		limitprintf (30,
			("ottointr: xmtqueues.pktswaiting went negative\n"));
		sc->data.xmtqueues.pktswaiting = 0;
		sc->data.xmtqueues.byteswaiting = 0;
	}
	OTTO_IFSTART (&sc->is_if);	/* queue output packets */

	sc->data.otto_watchdog.gotintr = sc->data.otto_watchdog.time;
	OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);
}

/* These routines are used by the fc interface.
   All are called with the softc lock held.
   See the header file for a complete description */
static void sanerange (unsigned *first, unsigned *last)
{
	if (*first < 1) {
		*first = 1;
	}
	if (*last >= OTTO_MAXVC) {
		*last = OTTO_MAXVC-1;
	}
}
/* acquire the interface's lock */
static fc_locktmp_t otto_fc_lock (struct otto_softc *sc)
{
	int s;
	OTTO_SMP_LOCK (&sc->lk_otto_softc, s);
	return (s);
}
/* release the interface's lock */
static void otto_fc_unlock (struct otto_softc *sc, fc_locktmp_t s)
{
	OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);
}
/* otto_fc_mstowait is supposed to return the time in milliseonds the
   resync code must wait for acks to drain on flow controlled circuits
   in range [first,last]. 
   OTTO doesn't allow such a figure to be calculated, so this procedure
   returns FC_INFINITE_MS until it can be sure all the pendings ACKs
   have gone, and then it returns 0.  

   This is the action on each call of otto_fc_mstowait:
   On the first call for a given request (i.e. firstcall is non-zero):
	Read the ACK FIFO pointers; set R and W to the read and write
	pointer values.  R and W are stored in the fc_ackstate
	structure.
	If R==W, there are no outstanding acks, return 0.
	Otherwise, call otto_fc_mstowait recursively so that we give
	prompt service when only one or two ACK are in the FIFO.
	If a call to otto_fc_mstowait returns 0, return 0.
	After two or threee tries, return whatever the recursive
	call on otto_fc_mstowait returned.
   On subsequent calls:
   	If any previous call returned 0, return 0 again.
	Read the ACK FIFO pointers; set R' and W' to the read ans write
	pointer values.
	if R'==W' or R'==W any previous ACKs have drained; return 0.

	Now the fifo is circular, so any simple tranlation of
	fifo pointer readings is equivalent to any other.
	Define a translation function f(x) to be
	((x)+OTTO_ACKFIFO_MOD-1-W) mod OTTO_ACKFIFO_MOD).  This
	function translates any fifo reading so that f(W) is
	the highest possible value: OTTO_ACKFIFO_MOD-1.  Thus
	forall x: f(x) <= f(W).  Since we know that R != W and
	R' != W, we have f(R) != f(W) and f(R') != f(W).
	Therefore, f(R) < f(W) and f(R') < f(W).  Since the
	pointers can only advance, if f(R') < f(R), the read
	pointer must have wrapped past the end of the number
	space, and therefore past the write pointer.

	Thus if f(R') < f(R) then any previous acks have
	drained; return 0.

	Otherwise either R'==R (no progress detected) or
	f(R) < f(R') < f(W).
	Set R to be the value of R' in the fc_ackstate structure,
	so that we will be more likely to detect a wrap next time.

	return FC_INFINITE_MS to indicate that there may still
	be pending ACKs and we don't know how long they will
	take to drain.
   */
static unsigned otto_fc_mstowait (struct otto_softc *sc, unsigned first, 
	unsigned last, unsigned firstcall, struct fc_ackstate *as)
{
#define OTTO_FC_READ 0
#define OTTO_FC_WRITE 1
#define OTTO_FC_NORM(_x,_w) (((_x)+2*OTTO_ACKFIFO_MOD-(_w)-1)%OTTO_ACKFIFO_MOD)

	unsigned rptr;
	unsigned wptr;
	unsigned orptr;
	unsigned owptr;
	if (firstcall) {
		unsigned i;
		otto_ackfifoptrs (&sc->data, &rptr, &wptr);
		as->uu[OTTO_FC_READ].ulng = rptr;
		as->uu[OTTO_FC_WRITE].ulng = wptr;
		if (rptr == wptr) {
			return (0);
		}
		for (i = 0; i != 2; i++) {	
			/* recurse: this loop makes it very likely that 
			   we'll return 0 after a few microseconds */
			if (otto_fc_mstowait (sc, first, last, 0, as) == 0) {
				return (0);
			}
		}
		/* tail recursion; fall through */
	}
	orptr = as->uu[OTTO_FC_READ].ulng;
	owptr = as->uu[OTTO_FC_WRITE].ulng;
	if (orptr == owptr) {	/* a previous call returned 0 */
		return (0);
	}
	otto_ackfifoptrs (&sc->data, &rptr, &wptr);
	if (rptr == wptr || rptr == owptr) {
		as->uu[OTTO_FC_READ].ulng = owptr; /* remember we returned 0 */
		return (0);
	}
	if (OTTO_FC_NORM (rptr, owptr) < OTTO_FC_NORM (rptr, owptr)) {
		as->uu[OTTO_FC_READ].ulng = owptr; /* remember we returned 0 */
		return (0);
	}
	as->uu[OTTO_FC_READ].ulng = rptr;
	return (FC_INFINITE_MS);
}
/* stop circuits that are flow controlled in range [first,last]*/
static void otto_fc_stop (struct otto_softc *sc, unsigned first, unsigned last)
{
	unsigned i;
	struct otto_vcdata *vcs = sc->data.vcs;

	ottodprintf (5, ("otto_fc_stop %u %u\n", first, last));
	if (sc->data.snmp.pkt.resyncVCI ==0) {
		/* don't stop the circuit if the far end doesn't
		   do flow control */
		return;
	}
	sanerange (&first, &last);
	for (i = first; i <= last; i++) {
		if ((vcs[i].flags & (OTTO_VC_FC|OTTO_VC_SCHED))==OTTO_VC_FC) {
			vcs[i].flags |= OTTO_VC_STOPPED;
			otto_settxvc (&sc->data, i, 1, -1, -1, -1);
		}
	}
}
/* start circuits that are flow controlled in range [first,last] */
static void otto_fc_start (struct otto_softc *sc, unsigned first, unsigned last,
	unsigned adjustcredits)
{
	unsigned i;
	struct otto_vcdata *vcs = sc->data.vcs;
	ottodprintf (5, ("otto_fc_start %u %u  adj%d\n", first,last,adjustcredits));
	sanerange (&first, &last);
	for (i = first; i <= last; i++) {
		if ((vcs[i].flags&(OTTO_VC_FC|OTTO_VC_SCHED|OTTO_VC_STOPPED)) ==
				(OTTO_VC_FC|OTTO_VC_STOPPED)) {
			otto_settxvc (&sc->data, i, 0, -1, -1,
				adjustcredits? 20 : -1);
		}
		vcs[i].flags &= ~OTTO_VC_STOPPED;
	}
}
/* send a packet on the flow control resync circuit */
static void otto_fc_send (struct otto_softc *sc, unsigned char *p, unsigned l)
{
	struct sockaddr dst;
	if (sc->data.snmp.pkt.resyncVCI == 0) {
		/* don't send the packet if the far end doesn't do flow
		   control.  */
		return;
	}
	dst.sa_family = AF_UNSPEC;
	/* addresses don't matter - this is null encapsulation.
	   Type must be 0. */
	bzero ((char *) dst.sa_data, 14);
	otto_simplesendpkt (sc, sc->data.snmp.pkt.resyncVCI, p, l, &dst);
}
/* Cause carrier to drop for at least a second */
static void otto_fc_zaplink (struct otto_softc *sc)
{
	otto_resetsuni (&sc->data);
	sc->data.otto_watchdog.resetsuni = 2;
}
/* end of routines used by the fc interface */




#define	ac ((struct arpcom *)ifp)

/* Interface exists: make available by filling in network interface
   record.  System will initialize the interface when it is ready
   to accept packets.  */
ottoattach (ui)
struct CONTROLLER_STRUCT *ui;
{
	int unit = ui->UNIT_NUM;
	struct otto_softc *sc = &otto_softc[unit];
	struct ifnet *ifp = &sc->is_if;
	struct sockaddr_in *sin;

	ifp->if_unit = unit;
	ifp->if_name = "otto";
	ifp->if_mtu = 9180;
#ifdef __osf__
	/* ifp->if_mediamtu = 65535;  Only in OSF 2.0 and later */
	ifp->if_addrlen = 6;
	ifp->if_hdrlen = 2 + sizeof(struct ether_header) + 8;
	ac->ac_bcastaddr = etherbroadcastaddr;
	ac->ac_arphrd = ARPHRD_ETHER;
#endif

	ifp->if_flags |= (IFF_BROADCAST | OTTO_IFF_MULTICAST |
#ifdef ultrix
			IFF_802HDR |
#endif
			OTTO_IFF_DYNPROTO | IFF_NOTRAILERS);
	ifp->if_type = OTTO_IFTYPE;

#ifndef  __osf__
	ifp->lk_softc = &sc->data.lk_otto_softc;
#endif 
#ifdef AC_IPUP
        ((struct arpcom *)ifp)->ac_flag = 0;
#else
	((struct arpcom *)ifp)->ac_ipaddr.s_addr = 0;
#endif
	ifp->if_sysid_type = 94; 	/* = Ethernet on a 3max */
					/* I have no idea what it should be */
					/* this number is used only by MOP */

		
	bcopy ((char *) &sc->data.physaddress.ether_addr_octet[0],
		(char *) &sc->is_addr[0], 6);

	/*
	 * attach interface
	 */
	sin = (struct sockaddr_in *)&ifp->if_addr;
	sin->sin_family = AF_INET;
	ifp->if_output = ottooutput;
	ifp->if_start = otto_ifstart;
#ifdef __osf__
	ifp->if_version = "SRC OTTO AN2 Interface";
#ifdef AC_IPUP
	/*XXX - this is a hack to get around the test in dli/dli_input.c
	  which assumes that all interfaces that want to use DLI
	  use ether_output as their output routine. XXX*/
	ifp->if_funhide = &sc->data.ifunhide;
	sc->data.ifunhide.if_output = ether_output;
#endif
#else
	ifp->d_affinity = ALLCPU;
	bcopy ("SRC OTTO AN2 Interface", ifp->if_version, 22);
	ifp->if_version[23] = '\0';
#endif
	ifp->if_init = ottoinit;
	ifp->if_ioctl = ottoifioctl;
	ifp->if_reset = ottoreset;
	ifp->if_watchdog = ottowatch;
	ifp->if_timer = 1;
	ifp->if_flags &= ~(IFF_OACTIVE);

	/*
	 * Hook up the device interrupt vector to begin allowing interrupts.
	 * We know that the device has been successfully initialized at this
	 * point. 
	 */

	printf("otto%d: %s, hardware address %s\n", 
		ifp->if_unit, ifp->if_version,
		ether_sprintf(sc->data.physaddress.ether_addr_octet));
#if (NPFWD > 0)
	pfwd_register(ifp, ottobridging, (char *)sc);
#endif
#if (NPACKETFILTER > 0) || defined(__osf__)
	attachpfilter(&(sc->is_ed));
#endif

	if_attach(ifp);
}


/* OTTO start routine. Strings output packets onto the transmit ring. */
/* call with softc lock held */
#ifdef __osf__
otto_ifstart (ifp)
struct ifnet *ifp;
{
	struct otto_softc *sc = &otto_softc[ifp->if_unit];
#else
otto_ifstart (unit)
int unit;
{
	struct otto_softc *sc = &otto_softc[unit];
	struct ifnet *ifp = &sc->is_if;
#endif
	struct otto_xmtq *xmtq;		/* xmt queue to use */

	/* Process transmit queues.  */
	while ((xmtq = sc->data.xmtqueues.ready) != (struct otto_xmtq *) 0 &&
	       ((sc->data.xmtqueues.pktswaiting < OTTO_MAXXMTQUEUED &&
	         sc->data.xmtqueues.byteswaiting < OTTO_MAXXMTBYTESQUEUED) ||
	        xmtq->next == &sc->data.xmtqueues.per_q[OTTO_IFQ_SPECIAL])) {
		struct mbuf *m0;	/* mbuf from queue */
		unsigned pktindex;	/* transmit pkt index */
		int copied;		/* whether packet has been copied */
		int totallen;		/* total length of packet */
		struct mbuf *m;		/* iteration variable */ 
		unsigned vc;		/* virtual circuit to use */
		int nfrags;		/* number of physical
				           fragments in packet */
	        unsigned descindex;
	        unsigned pktdescindex;
		volatile struct otto_descriptor *desc;
					/* transmit descriptor followed
					   by OTTO_MAXFRAGSPERTXBUF fragment
					   descriptors */
		struct otto_txpkt *txpkt;	/* transmit pkt info */
		unsigned *txvcpkt;
		unsigned lastpkt;
		int physlen;		/* bytes a in physical fragment */
		struct otto_xmtq *nextxmtq;	/* next xmtq to try */

		if ((pktindex = otto_alloctxpkt (&sc->data)) == 0) {
			/* give up if adapter runs out of descriptors */
			goto no_desc;
		}

		nextxmtq = xmtq->next;
		IF_DEQUEUE (nextxmtq->pifq, m0); /* get packet to send */
		sc->data.xmtqueues.totallen--;

		copied = 0;		/* packet not copied yet */

		/* get the VC to use */
#ifdef __alpha
                vc = *(unsigned *)&m0->m_ipq[0];
#else
                vc = m0->m_context;
#endif

		txpkt = &sc->data.txpkts[pktindex];
		txpkt->q = nextxmtq - &sc->data.xmtqueues.per_q[0];

retry:		/* return here if we had to copy the packet into a new mbuf */

		/* The fragment descriptors immediately follow the packet
		   descriptor.   The variable descindex is the index of the 
		   *next* fragment descriptor (its the bit pattern to be put into
		   the current fragment to make it point to the next one).
		   The variable desc starts pointing at the packet descriptor,
		   but then is incremented to point to the *current* fragment
		   descriptor on each iteration of the loop below. */

		pktdescindex = txpkt->desc;
		desc = &sc->data.descriptors[pktdescindex];	/* pkt desc */
		descindex = pktdescindex+1;	/* index of frag desc */

		desc->word0 = OTTO_MAKE_TXPKT0 (OTTO_TXPKT_DOCRC, vc, descindex);
		desc->word1 = OTTO_MAKE_TXPKT1 (OTTO_TXACTION_SEND);

		descindex++;		/* index of second frag desc */
		desc++;			/* ptr to first frag desc */

		/* zero the AAL5 trailer */
		bzero ((char *) txpkt->trailerv, OTTO_MAXTRAILERLEN);

		/* Deal with each mbuf in chain.  One mbuf can be multiple
		   physical fragments because NFS code produces mbufs
		   that span non-contiguous physical pages.  */
		m = m0;
		nfrags = 0;
		totallen = 0;
		while (m != (struct mbuf *) 0) {
			caddr_t ptr;	/* address of mbuf data */
			int loworder;	/* offset within page of mbuf data */
			int bytesleft;	/* bytes not processed in this mbuf */
			struct mbuf *n = m->m_next;

			/* get pointer to data in mbuf */
			ptr = mtod (m, caddr_t);

			/* loworder is offset of data with a page */
			loworder = ((int) ptr) & PGOFSET;	

			bytesleft = m->m_len;
			totallen += bytesleft;

			while (bytesleft != 0) { /* while bytes to process */
				otto_uword32 physaddr;	/* physical address */

				if (loworder + bytesleft > NBPG) {
					/* mbuf spans pages */
					physlen = NBPG - loworder;
					loworder = 0;
					bytesleft -= physlen;
				} else {
					/* rest of mbuf is on one page */
					physlen = bytesleft;
					bytesleft = 0;
				}

				/* Must copy the packet if there are
				   too many fragments, or if any fragment
				   doesn't start/end on a 4 byte boundary.
				   This test leaves one fragment free for
				   AAL5 trailer.  */
				if (nfrags == OTTO_MAXFRAGSPERTXBUF-1) {
					OTTO_STATS_INC (&sc->data,
						xmt_too_many_xmt_frags);
					ottodprintf (3, ("2mf: %d\n", nfrags));
					goto copy;
				} else if ((loworder & 3) != 0) {
					OTTO_STATS_INC (&sc->data,
						xmt_bad_aligned_frag);
					ottodprintf (3, ("align: %x\n", loworder));
					goto copy;
				} else if ((physlen & 3) != 0) {
					if (n == (struct mbuf *) 0 && 
						bytesleft == 0) {
						int runt = (physlen & 3);
					    	bcopy ((char *)ptr+physlen-runt,
						 (char *)txpkt->trailerv, runt);
					    	physlen -= runt;
						ottodprintf (3, ("runt: %d\n", runt));
					} else {
						OTTO_STATS_INC (&sc->data,
							xmt_bad_len_frag);
						ottodprintf (3, ("len: %x\n", physlen));
						goto copy;
					}
				}

				if (physlen > 0) {
					physaddr = (otto_uword32)
						OTTO_SVTOPHY(ptr);
					ptr += physlen;
					nfrags++;

					desc->word0 = OTTO_MAKE_FRAG0 (physlen,
						descindex);
					desc->word1 =OTTO_MAKE_FRAG1 (physaddr);
					descindex++;
					desc++;
				}
			}

			m = n;
		}

		/* add the trailer */
		physlen = (totallen + OTTO_MINTRAILERLEN + OTTO_PKTROUNDUP-1) /
				OTTO_PKTROUNDUP;	/* cells needed */
		physlen *= OTTO_PKTROUNDUP;		/* bytes on wire */
		physlen -= ((totallen & ~3) + OTTO_CRCLEN);
							/* bytes to be 
						   	   added by driver */
		/* put length in AAL5 trailer. */
	        txpkt->trailerv[physlen-2] = totallen>>8;
	        txpkt->trailerv[physlen-1] = totallen;

	        totallen &= ~3;		/* make totallen include trailer */
	        totallen += physlen + OTTO_CRCLEN;

		if ((totallen % OTTO_PKTROUNDUP) != 0) {
			ottodprintf (0, ("bad length sent\n"));
		}

		desc->word0 = OTTO_MAKE_FRAG0 (physlen, OTTO_FINALFRAG);
		desc->word1 = OTTO_MAKE_FRAG1 (txpkt->trailerp);
		descindex++;
		desc++;

		/* Need only memory barriers here, but have
		   to use a write-buffer flush on mips machines.  */
		wbflush ();
		*sc->data.txrequest =
			OTTO_MAKE_TXREQ (pktdescindex, totallen - OTTO_CRCLEN);
		wbflush ();

		/* record mbuf so it can be freed in interrupt routine */
		txpkt->m = m0;
		txpkt->len = totallen;

		/* Link packet to chain of those to be transmitted on this
		   VC.  This allows us to find them given the VCI in
		   the transmit report. */
		txvcpkt = &sc->data.vcs[vc].txpkts;
		lastpkt = *txvcpkt;
		if (lastpkt == 0) {
			txpkt->next = pktindex;
		} else {
			struct otto_txpkt *lasttxpkt= &sc->data.txpkts[lastpkt];
			txpkt->next = lasttxpkt->next;
			lasttxpkt->next = pktindex;
		}
		*txvcpkt = pktindex;

		{
			unsigned flags = sc->data.vcs[vc].flags;
			if ((flags & OTTO_VC_XMTSTART) == 0) {
				unsigned nxmtvcs = sc->data.nxmtvcs;
				if (nxmtvcs < OTTO_NTXPKTS) {
					sc->data.xmtvcs[nxmtvcs] = vc;
					sc->data.nxmtvcs = nxmtvcs+1;
				}
				sc->data.vcs[vc].flags = flags|OTTO_VC_XMTSTART;
			}
		}

		/* dead-man's handle */
		sc->data.otto_watchdog.queuedxmtbuf =
			sc->data.otto_watchdog.time;

		otto_event (&sc->data, vc, totallen, OTTO_EVENT_TX);

		/* DECnet statistics */
		if (sc->is_ctrblk.est_bytesent+totallen >
				sc->is_ctrblk.est_bytesent) {
			sc->is_ctrblk.est_bytesent += totallen;
		}
		if (sc->is_ctrblk.est_bloksent != ~0) {
			sc->is_ctrblk.est_bloksent++;
		}
		if ((mtod (m0, u_char *)[2]) & 1) {	/*XXXmulticast packets*/
			if (sc->is_ctrblk.est_mbytesent+totallen >
					sc->is_ctrblk.est_mbytesent) {
				sc->is_ctrblk.est_mbytesent += totallen;
			}
			if (sc->is_ctrblk.est_mbloksent != ~0) {
				sc->is_ctrblk.est_mbloksent++;
			}
		}

		sc->data.xmtqueues.pktswaiting++;
		sc->data.xmtqueues.byteswaiting += txpkt->len;
		nextxmtq->pktswaiting++;
		if (nextxmtq->pifq->ifq_head &&
		    nextxmtq->pktswaiting < nextxmtq->maxpktswaiting) {
			/* keep this queue on the ready list */
			sc->data.xmtqueues.ready = nextxmtq;
	        } else { /* remove queue from ready list */
	        	if (xmtq == nextxmtq) {	/* list is now empty */
	        		sc->data.xmtqueues.ready=(struct otto_xmtq *)0;
	        	} else {
	        		xmtq->next = nextxmtq->next;
	        	}
	        	nextxmtq->next = (struct otto_xmtq *) 0;
	        }
		continue;
copy:		
		if (!copied) {
			m = copypacket (sc, m0, M_COPYALL, 0);
			copied = 1;
		} else {
			m = (struct mbuf *) 0;
			copied = 2;
		}
		OTTOMFREEM (m0);
		if (m != (struct mbuf *) 0) {
			m0 = m;
			goto retry;
		}
		limitprintf (5,
			("otto_ifstart: copypacket failed (copied=%d)\n",
				copied-1));
		otto_freetxpkt (&sc->data, pktindex);
	}
	if (sc->data.xmtqueues.pktswaiting && !(ifp->if_flags & IFF_OACTIVE)) {
		ifp->if_flags |= IFF_OACTIVE;
	}
	return;

no_desc:
	limitprintf (5, ("otto_ifstart: no fragments\n"));
	OTTO_STATS_INC (&sc->data, no_free_desc);
}

/* Initialize interface. */
ottoinit(unit)
int unit;
{
	struct otto_softc *sc = &otto_softc[unit];
	struct ifnet *ifp = &sc->is_if;
	OTTO_SMP_LOCKTMP s;
	int working, count;

	/* do nothing if address still unknown */
	if (ifp->if_addrlist == (struct ifaddr *)0) {
		return;
	}
	if (ifp->if_flags & IFF_RUNNING) {
		return;
	}

	/* Lock the softc */
	OTTO_SMP_LOCK (&sc->lk_otto_softc, s);

	/* reinitialize the device and the free lists for its resources */
	if (otto_softreset (&sc->data, OTTO_TXCELLS, OTTO_NOACKCELLS) != 0) {
		panic ("otto_softreset failed");
	}

	/* Mark interface up and running; start output on device.  */
	sc->is_if.if_flags |= (IFF_UP|IFF_RUNNING);
	sc->is_if.if_flags &= ~IFF_OACTIVE; 

	sc->ztime = time.tv_sec;

	initsoftc (sc);

	OTTO_IFSTART (&sc->is_if);		/* queue output packets */

	/* unlock softc, drop IPL */
	OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);

	/* wait for up to 30 seconds for network to come up */
	working = 0;
	for (count = 0; count < 30; count++) {
		extern void wakeup ();

		OTTO_SMP_LOCK (&sc->lk_otto_softc, s);
		working = /* is at least one of the links working */1;
		OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);

		if (working) {
			if (count > 3) {
				cprintf ("found an ATM switch\n");
			}
			return;
		}
		if (count == 3) {
			cprintf ("trying to contact ATM switch --- will try for 30 seconds\n");
		}
		/* we use our own lbolt location here because the global
		 * lbolt isn't being woken up during initialization,
		 * and this routine needs to work early on for 
		 * discless machines.
		 */
		timeout (wakeup, (caddr_t) &sc->data.lbolt, hz);
		sleep ((caddr_t) &sc->data.lbolt, PZERO);
	}
	cprintf ("failed to find an ATM switch\n");
}

/* Check for stall transmits on VCs.   
   Called with softc lock held */
static int checkstall (sc, vc)
struct otto_softc *sc;
unsigned vc;
{
	struct otto_vcdata *vcdata = &sc->data.vcs[vc];
	unsigned flags = vcdata->flags;

	if (vcdata->txpkts == 0) {
		if ((flags & OTTO_VC_XMTFLUSH) != 0) {
			otto_settxvc (&sc->data, vc, flags & OTTO_VC_SCHED,
				flags & OTTO_VC_FC, -1, -1);
			if (flags & OTTO_VC_FC) {
				fc_resync (sc->data.fcs, vc, vc, 0);
			}
		}
		/* can't use "flags" variable here; it may not be up-to-date
		   because fc_resync may have changed it. */
		vcdata->flags &= ~(OTTO_VC_XMTSTART|OTTO_VC_XMTDONE|
				OTTO_VC_XMTPEND|OTTO_VC_XMTFLUSH);
		return (0);
	}
	if ((flags & OTTO_VC_XMTDONE) != 0) {
		flags &= ~(OTTO_VC_XMTDONE|OTTO_VC_XMTPEND);
#if 0
/* plain-jane otto driver lets a VC without credit send its cells anyway
   if it's been stalled a long time w/o credit, as this is the only way to
   reclaim on-board buffer space for that VC's queued cells. We'll bag on
   this for now, with the possible consequence that credit starvation on
   one VC can keep us from sending cells on _any_ VC */
	} else if ((flags & OTTO_VC_XMTPEND) != 0) {
		if ((flags & OTTO_VC_STOPPED) == 0) {
			/* VC has been stalled for a long time.
			   Turn off both flow control and schedule to
			   get back transmit buffers. */
			cprintf ("otto: transmit VC %d stalled; flushing\n",vc);
			otto_settxvc (&sc->data, vc, 0, 0, 0, -1);
			flags |= OTTO_VC_XMTFLUSH;
		}
#endif
	} else {
		flags |= OTTO_VC_XMTPEND;
	}
	vcdata->flags = flags | OTTO_VC_XMTSTART;
	return (1);
}


int ottogetvc = 0;

ottowatch (unit)
int unit;
{
	struct otto_softc *sc = &otto_softc[unit];
	struct ifnet *ifp = &sc->is_if;
	int reset = 0;
	int fakeinterrupt = 0;
	unsigned t;
	OTTO_SMP_LOCKTMP s;
	unsigned rcvpkt = 1;

	if ((ifp->if_flags & IFF_RUNNING) && (ifp->if_flags & IFF_UP)) {
		t = sc->data.otto_watchdog.time;
		sc->data.otto_watchdog.time = t+1;

		if (ottogetvc) {
			otto_printvc (&sc->data, ottogetvc);
			ottogetvc = 0;
		}

		if (t != sc->data.otto_watchdog.gotintr) {
			fakeinterrupt = 1;
		}
#ifdef notdef
		if (t - sc->data.otto_watchdog.linkworking > 10) {
			reset = 1;
		}
		if (sc->data.controller->link[0]->working ||
		    sc->data.controller->link[1]->working) {
			sc->data.otto_watchdog.linkworking = t;
		}
#endif
		if (t - sc->data.otto_watchdog.queuedrcvbuf > 1) {
			fakeinterrupt = 1;
		}
		if (t - sc->data.otto_watchdog.queuedxmtbuf > 1) {
			fakeinterrupt = 1;
		}
		if (t - sc->data.otto_watchdog.gotrcv > 1) {
			fakeinterrupt = 1;
			rcvpkt = 0;
		}
		if (t - sc->data.otto_watchdog.gotxmt > 5) {
		}
		if (sc->data.otto_watchdog.forcereset) {
			reset = 1;
		}
		if (sc->data.ram[OTTO_CRASHFLAG] == OTTO_CRASHVALUE) {
			unsigned i;
			reset = 1;
			printf ("otto crash:\n");
			for (i = 16; i < 60; i++) {
				unsigned r0 = sc->data.ram[0x7000+i] & 0xFFFF;
				if (r0 != 0) {
					printf("C %2d: %4x %6d\n",
						i, r0, r0 & 0x1FFF);
				}
			}
			printf("Transmit DMA Queue:\n");
			for (i = OTTO_DMAFFPOS;
			      i < OTTO_DMAFFPOS+OTTO_DMAFFSIZE; i++) {
				otto_uword32 temp = sc->data.ram[i];
				if (temp != 0) {
					printf("%4x/ %8x\n", i, temp);
				}
			}
		}

		OTTO_SMP_LOCK (&sc->lk_otto_softc, s);

		otto_event (&sc->data, 0, 0, OTTO_EVENT_TICK);

		if (sc->data.otto_watchdog.resetsuni != 0 ||
		    !otto_gettingcells (&sc->data)) {
			if (!sc->data.otto_watchdog.notgettingcells) {
				otto_fakesnmp_updown (sc, 0);
				otto_setled (&sc->data, sc->data.linkinuse, 0);
				sc->data.arpstate.rearp = 10;
				sc->data.otto_watchdog.notgettingcells = 1;
			}
			/* turn on SUNI after a reset */
			if (sc->data.otto_watchdog.resetsuni != 0) {
				sc->data.otto_watchdog.resetsuni--;
				if (sc->data.otto_watchdog.resetsuni == 0) {
					otto_initsuni (&sc->data);
				}
			}
		} else {
			if (sc->data.otto_watchdog.notgettingcells) {
				otto_fakesnmp_updown (sc, 1);
				sc->data.otto_watchdog.notgettingcells = 0;
			}
			if (rcvpkt) {
				otto_setled (&sc->data, sc->data.linkinuse, 1);
			} else {
				otto_setled (&sc->data, sc->data.linkinuse, 2);
			}
		}
		/* check for stalled transmissions */
		if ((t & 0x7) == 0) {
			unsigned nxmtvcs = sc->data.nxmtvcs;
			unsigned keep = 0;
			unsigned vc;
			if (nxmtvcs == OTTO_NTXPKTS) {
				for (vc = 1; vc != OTTO_MAXVC; vc++) {
					if (checkstall (sc, vc) &&
					    keep != OTTO_NTXPKTS) {
						sc->data.xmtvcs[keep++] = vc;
					}
				}
			} else {
				unsigned check;
				for (check = 0; check != nxmtvcs; check++) {
					vc = sc->data.xmtvcs[check];
					if (checkstall (sc, vc)) {
						sc->data.xmtvcs[keep++] = vc;
					}
				}
			}
			sc->data.nxmtvcs = keep;
		}

		otto_arp_tick (sc);		/* handle ARPs */
		otto_fakesnmp_tick (sc);	/* handle fake SNMP */

		{
			unsigned lasttime = 0;
			unsigned nexttime;
			while ((nexttime = sc->data.ringmissedlist) != 0 &&
					nexttime != lasttime) {
				queuercvbuffers (sc, nexttime, 0,
					(struct mbuf *) 0, 0);
				lasttime = nexttime;
			}
		}
		OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);

		if (reset) {
			if (t - sc->data.otto_watchdog.reset > 25) {
				ottorestart (unit);
				sc->data.otto_watchdog.reset = t;
				sc->data.otto_watchdog.forcereset = 0;
			}
		} else if (fakeinterrupt) {
			ottointr (unit);
		}
	}
	
	ifp->if_timer = 1;		/* call again in a second */
}


/* free mbufs on controller's queues.
   The controller must be quiescent when this is called */
static void freequeuedmbufs (sc)
struct otto_softc *sc;
{
	int i;

	/* fix the report ring so that if the interrupt system happens
	   to have a pending interrupt for us, the interrupt routine
	   won't do anything */
	sc->data.reportbase[0].dmareport = OTTO_REP_DMA_TAG;
	sc->data.nextreport = 0;
	sc->data.reporttag = OTTO_REP_DMA_NEGTAG;

	for (i = 0; i != OTTO_NRXPKTS; i++) {
		if (sc->data.rxpkts[i].m != (struct mbuf *) 0) {
			OTTOMFREEM (sc->data.rxpkts[i].m);
			sc->data.rxpkts[i].m = (struct mbuf *) 0;
		}
	}
	for (i = 0; i != OTTO_NTXPKTS; i++) {
		if (sc->data.txpkts[i].m != (struct mbuf *) 0) {
			OTTOMFREEM (sc->data.txpkts[i].m);
			sc->data.txpkts[i].m = (struct mbuf *) 0;
		}
	}
}


/* Copy len bytes from mbuf chain into a chain of mbuf clusters.
   The first off bytes are skipped.  The result chain contains only
   mbuf clusters, and is densely packed */
static struct mbuf *copypacket (sc, m, len, off)
struct otto_softc *sc;
struct mbuf *m;
unsigned len;
unsigned off;
{
	int newused = M_CLUSTERSZ;
	int oldused = 0;
	struct mbuf *newchain = 0;
	struct mbuf *n;
	unsigned j;

	/* skip the first "off" bytes */
	while (m != 0 && off != 0) {
		if (off >= m->m_len) {
			off -= m->m_len;
			m = m->m_next;
		} else {
			oldused = off;
			off = 0;
		}
	}

	while (m != 0 && len != 0) {
		j = m->m_len - oldused;		/* calculate how much to copy */

		/* allocate a new cluster if need more room */
		if (j != 0 && newused == M_CLUSTERSZ) {
			struct mbuf *nextn;

			OTTOMCLGET (nextn);
			if (nextn == 0) {
				limitprintf (5, ("copypacket: can't get cluster\n"));
				sc->is_ctrblk.est_sysbuf++;
				OTTOMFREEM (newchain);
				return ((struct mbuf *) 0);
			} else {
				if (newchain == 0) {
					newchain = nextn;
				} else {
					/* n is not used before set here */
					n->m_next = nextn;
				}
				n = nextn;
				newused = 0;
			}
		}

		/* don't overfill cluster, or copy more than requested */
		if (j > M_CLUSTERSZ - newused) {
			j = M_CLUSTERSZ - newused;
		}
		if (j > len) {
			j = len;
		}

		if (j != 0) {
			bcopy (mtod (m, caddr_t) + oldused,
			       mtod (n, caddr_t) + newused, (int) j);
		}
		newused += j;
		oldused += j;
		len -= j;

		/* move on to next mbuf in chain if needed */
		if (oldused == m->m_len) {
			oldused = 0;
			m = m->m_next;
		}
	}
	if (newchain != 0) {
		n->m_len = newused;
	}

	return (newchain);
}


/* initialize ring q. */
static void initring (sc, q)
struct otto_softc *sc;
unsigned q;
{
	struct otto_ring *ring = &sc->data.rings[q];
	volatile struct otto_descriptor *ringdesc =
		&sc->data.descriptors[ring->desc];
	int i;

	ring->tag = OTTO_TAG0;
	ring->nextfree = 0;
	ring->lastused = 0;
	ring->missed = 0;
	ringdesc[0].word0 = OTTO_MAKE_RING (OTTO_TAG0, ring->desc+1);
	for (i = 1; i != OTTO_MAXPKTSPERRING; i++) {
		ringdesc[i].word0 = OTTO_MAKE_RXPKT (OTTO_DONTWRAP,
			ring->desc+i+1, OTTO_TAG1, OTTO_FINALFRAG);
		ring->pktindex[i] = 0;
	}
	ringdesc[OTTO_MAXPKTSPERRING].word0 = OTTO_MAKE_RXPKT (OTTO_DOWRAP,
			ring->desc+1, OTTO_TAG1, OTTO_FINALFRAG);
	ring->pktindex[0] = 0;
}

/* The receive buffer rxpkt has an existing tail of mbufs m, but  n mbufs
   are missing from the front of the receive buffer.   This routine
   adds the missing mbufs to make a complete chain.   If it succeeds
   it returns 1, otherwise it frees m, and rxpkt, and returns 0. */
static int replenishrcvbuffer (sc, rxpkt, n, m)
struct otto_softc *sc;
struct otto_rxpkt *rxpkt;
int n;
struct mbuf *m;
{
	unsigned rxfragdescindex = rxpkt->desc+n;
	volatile struct otto_descriptor *rxfragdesc = 
		&sc->data.descriptors[rxfragdescindex-1];

	while (n != 0) {
		struct mbuf *mtmp;
		otto_uword32 addr;

		OTTOMCLGET (mtmp);
		/* get can't a cluster, give up */
		if (mtmp == (struct mbuf *) 0) {
			goto no_mbuf;
		}

		addr = (otto_uword32) OTTO_SVTOPHY (mtod (mtmp, caddr_t));
#ifndef __osf__
		/* save the physical address in the ifp field */
		mtmp->m_ifp = (struct ifnet *) addr;
#endif
		ottodprintf (3, ("replenish phys=%x\n", addr));

		/*XXX shouldn't have to set the first word of the
		  fragment descriptor here.   This would save a PIO
		  write per fragment. */
		if (m == (struct mbuf *) 0) {
			rxfragdesc->word0 = OTTO_MAKE_FRAG0 (
				M_CLUSTERSZ, OTTO_FINALFRAG);
		} else {
			rxfragdesc->word0 = OTTO_MAKE_FRAG0 (
				M_CLUSTERSZ, rxfragdescindex);
		}
		rxfragdesc->word1 = OTTO_MAKE_FRAG1 (addr);
		rxfragdesc--;
		rxfragdescindex--;
		mtmp->m_next = m;
		m = mtmp;
		n--;
	}

	rxpkt->m = m;

	return (1);

no_mbuf:
	sc->is_ctrblk.est_sysbuf++;
	limitprintf (30, ("otto: can't get mbuf cluster - memory full?\n"));
	OTTOMFREEM (m);
	otto_freerxpkt (&sc->data, rxpkt - sc->data.rxpkts);
	return (0);
}

/* Queue "n" receive buffers on a ring.   mbuf clusters are be taken from 
   m0 if there are any there.  Unused clusters from that list are
   freed.  If there aren't enough clusters in the list, new ones are
   allocated. */
static void queuercvbuffers (sc, q, n, m0, canwait)
struct otto_softc *sc;
unsigned q;
int n;
struct mbuf *m0;
int canwait;
{
	struct otto_ring *ring = &sc->data.rings[q];
	unsigned nextfree = ring->nextfree;
	unsigned lastused = ring->lastused;
	unsigned tag = ring->tag;
	volatile struct otto_descriptor *rcvring =
		&sc->data.descriptors[ring->desc+1];
	int nnfree;
	int nexttag;
	struct mbuf *m;
	struct mbuf *mtmp;
	int i;
	volatile struct otto_descriptor *rxfragdesc;
	unsigned rxfragdescindex;
	unsigned rxpktindex;
	otto_uword32 addr;

	n += ring->missed;		/* catchup with missed buffers */

	while (n > 0) {
		nnfree = nextfree+1;
		nexttag = tag;
		if (nnfree == OTTO_MAXPKTSPERRING) {
			nnfree = 0;
			nexttag = OTTO_TAG_FLIP (tag);
		}

		/* if the next entry would fill the ring, give up */
		if (nnfree == lastused) {
			limitprintf (5, ("otto: queuercvbuffers: ring full\n"));
			break;
		}

		/* pull up to OTTO_CLPERRCVPKT mbufs off "m0" list
		   and push onto "m" list */
		m = (struct mbuf *) 0;
		i = 0;
		while (m0 != (struct mbuf *) 0 && i != OTTO_CLPERRCVPKT) {
			mtmp = m0;
			m0 = m0->m_next;
			mtmp->m_next = m;
			m = mtmp;
			i++;
		}

		while (i != OTTO_CLPERRCVPKT) {
			if (canwait) { 
				OTTOMCLGETWAIT (mtmp);
			} else {
				OTTOMCLGET (mtmp);
			}
			/* get can't a cluster, give up */
			if (mtmp == (struct mbuf *) 0) {
				sc->is_ctrblk.est_sysbuf++;
				limitprintf (30, ("otto: queuercvbuffers: can't get mbuf cluster - memory full?\n"));
				OTTOMFREEM (m);
				m = (struct mbuf *) 0;
				break;
			}
			mtmp->m_next = m;
			m = mtmp;
			i++;
		}

		if (m == (struct mbuf *) 0) {
			break;
		}

		rxpktindex = otto_allocrxpkt (&sc->data);
		/* if there are no free buffers, give up */
		if (rxpktindex == 0) {
			limitprintf (30, ("queuercvbuffers: no free buf descs\n"));
			OTTO_STATS_INC (&sc->data, no_free_desc);
			OTTOMFREEM (m);
			break;
		}
		sc->data.rxpkts[rxpktindex].m = m;

		rxfragdescindex = sc->data.rxpkts[rxpktindex].desc;
		tag = OTTO_MAKE_RXPKT (0, 0, tag, rxfragdescindex);
		rxfragdesc = &sc->data.descriptors[rxfragdescindex++];

		while (m != (struct mbuf *) 0) {
			mtmp = m->m_next;
			addr = (otto_uword32) OTTO_SVTOPHY (mtod (m, caddr_t));
#ifndef __osf__
			/* save the physical address in the ifp field */
			m->m_ifp = (struct ifnet *) addr;
#endif 
			if (mtmp == (struct mbuf *) 0) {
				rxfragdesc->word0 = OTTO_MAKE_FRAG0 (
					M_CLUSTERSZ, OTTO_FINALFRAG);
			} else {
				rxfragdesc->word0 = OTTO_MAKE_FRAG0 (
					M_CLUSTERSZ, rxfragdescindex);
			}
			rxfragdesc->word1 = OTTO_MAKE_FRAG1 (addr);
			rxfragdesc++;
			rxfragdescindex++;
			m = mtmp;
		}
		wbflush ();
		OTTO_WRITE_LOW (&rcvring[nextfree].word0, tag, 0xffff);
		wbflush ();

		ring->pktindex[nextfree] = rxpktindex;

		tag = nexttag;
		nextfree = nnfree;
		n--;

		/* dead man's handle */
		sc->data.otto_watchdog.queuedrcvbuf =
			sc->data.otto_watchdog.time;
	}

	if (n != 0 && ring->missed == 0) {
		ring->next = sc->data.ringmissedlist;
		sc->data.ringmissedlist = ring - sc->data.rings;
	} else if (ring->missed != 0 && n==0 && sc->data.ringmissedlist != 0) {
		struct otto_ring *x = &sc->data.rings[sc->data.ringmissedlist];
		if (ring == x) {
			sc->data.ringmissedlist = ring->next;
		} else {
			while (x->next != 0) {
				struct otto_ring *nx = &sc->data.rings[x->next];
				if (nx == ring) {
					x->next = ring->next;
					break;
				}
				x = nx;
			}
		}
	}
	ring->missed = n;
	ring->tag = tag;
	ring->nextfree = nextfree;

	OTTOMFREEM (m0);
}


/* do interface ioctls */
ottoifioctl (ifp, cmd, data)
struct ifnet *ifp;
unsigned cmd;
caddr_t data;
{
	int error = 0;
	struct otto_softc *sc = &otto_softc[ifp->if_unit];
	struct ifreq *ifr = (struct ifreq *)data;
	struct ifaddr *ifa = (struct ifaddr *)data;
	struct ifdevea *ifd = (struct ifdevea *)data;
	struct ctrreq *ctr = (struct ctrreq *)data;
	struct protosw *pr;
	OTTO_SMP_LOCKTMP s;
	int i;
	char *p;

	if (cmd != SIOCSIFADDR && cmd != SIOCIFRESET) {
		OTTO_SMP_LOCK (&sc->lk_otto_softc, s);
	}
	switch (cmd)
	{
	case SIOCSPHYSADDR: 
		bcopy ((char *) &ifr->ifr_addr.sa_data[0],
			    (char *) &sc->is_addr[0], 6);
#if (NPACKETFILTER > 0) || defined(__osf__)
		pfilt_newaddress (sc->is_ed.ess_enetunit, sc->is_addr);
#endif
		break;

	case SIOCRPHYSADDR: 
		bcopy ((char *)&sc->is_addr[0], (char *)&ifd->current_pa[0], 6);
		bcopy ((char *)&sc->data.physaddress.ether_addr_octet[0],
			(char *)&ifd->default_pa[0], 6);
		break;

		/*
		 * SIOCADDMULTI and SIOCDELMULTI are processed completely
		 * differently on osf and ultrix.
		 *
		 * On osf:
		 * - higher level keeps reference counts
		 * - ALLMULTI is aset in flags and called if necessary
		 * - ADDMULTI and DELMULTI called iff refcounts say so
		 *   for particular ethernet addresses.
		 *
		 * On ultrix:
		 * - you get called with an unmunged structure
		 * - you should call ether_addmulti
		 * - this allows the higher layers to keep the right refcounts
		 * - if you need to do anything ether_addmulti will return
		 *   ENETRESET in which case you must process the list.
		 * - if the high and low entries in the list are not the
		 *   same you must switch allmulti on.
		 */
	case SIOCDELMULTI:
		break;

	case SIOCADDMULTI:
		break;
		
	case SIOCSIFFLAGS:
		sc->data.ifflagscopy = ifp->if_flags;
		break;
	
	case SIOCENABLBACK:
		error = EOPNOTSUPP;
		break;
	case SIOCDISABLBACK:
		error = EOPNOTSUPP;
		break;
	
	case SIOCIFRESET:
		ottorestart (ifp->if_unit);
		break;

	case SIOCRDCTRS:
	case SIOCRDZCTRS:
                if (ctr->ctr_type == CTR_ETHER) {
                        ctr->ctr_ether = sc->is_ctrblk;
                        ctr->ctr_ether.est_seconds =
                                (time.tv_sec-sc->ztime) > 0xfffe? 0xffff :
                                        (time.tv_sec - sc->ztime);
                } else {
                        if (ctr->ctr_type == OTTO_CTR_LINK0) {
                                p = (char *) &sc->data.otto_stats.perlink[0];
                                i = sizeof (sc->data.otto_stats.perlink[0]);
                        } else if (ctr->ctr_type == OTTO_CTR_LINK1) {
                                p = (char *) &sc->data.otto_stats.perlink[1];
                                i = sizeof (sc->data.otto_stats.perlink[1]);
                        } else {
                                /* assume ctr->ctr_type == OTTO_CTR_BOTHLINKS */
                                p = (char *) &sc->data.otto_stats.bothlinks;
                                i = sizeof (sc->data.otto_stats.bothlinks);
                        }
                        bcopy ((char *) p, (char *) &ctr->ctr_ether, i);
                }
                if (cmd == SIOCRDZCTRS) {
                        if (ctr->ctr_type == CTR_ETHER) {
                                sc->ztime = time.tv_sec;
                                bzero ((char *) &sc->is_ctrblk,
                                        sizeof(struct estat));
                        } else {
                                bzero (p, i);
                        }
                }
                break;

	case SIOCSIFADDR:
		/*
		 * Init the interface if it's not already running
		 */

		ottoinit (ifp->if_unit);
#ifndef AC_IPUP
#ifdef __osf__
		switch (ifa->ifa_addr->sa_family)
#else 
		switch (ifa->ifa_addr.sa_family)
#endif
		{
#if defined(INET) || defined(__osf__)
		case AF_INET:
			OTTO_SMP_LOCK (&lk_ifnet, s);
			((struct arpcom *)ifp)->ac_ipaddr =
				IA_SIN(ifa)->sin_addr;
			OTTO_SMP_UNLOCK (&lk_ifnet, s);
			/* 1st packet out */
			arpwhohas((struct arpcom *)ifp, &IA_SIN(ifa)->sin_addr);
			break;
#endif
		default:
#ifndef __osf__
			if (pr=iffamily_to_proto(ifa->ifa_addr.sa_family)) {
				error = (*pr->pr_ifioctl)(ifp, cmd, data);
			}
#endif
			break;
		}
#endif
		break;
#ifdef SIOCSIPMTU
	case SIOCSIPMTU:
		bcopy (ifr->ifr_data, (char *) &(ifp->if_mtu),
			sizeof (ifp->if_mtu));
		ifp->if_mtu = (ifp->if_mtu>=IP_MINMTU)?  ifp->if_mtu: IP_MINMTU;
		break;
#endif
#ifdef SIOCRIPMTU
	case SIOCRIPMTU:
		bcopy ((char *) &(ifp->if_mtu), ifr->ifr_data,
			sizeof (ifp->if_mtu));
		break;
#endif
	default:
		cprintf ("ottoifioctl: ifp=%x cmd=%x data=%x\n", ifp, cmd, data);
		error = EINVAL;
		break;
	}
	if (cmd != SIOCSIFADDR && cmd != SIOCIFRESET) {
		OTTO_SMP_UNLOCK (&sc->lk_otto_softc, s);
	}
	return (error);
}


/* structure used to pass information from simplesend pkt when
   dst->sa_family = AF_OTTO */
#define AF_OTTO 255
struct sockaddr_otto {
	struct sockaddr dst;
	unsigned vc;
	struct otto_softc *sc;
};

/* OTTO output routine.  */
#ifdef __osf__
ottooutput(ifp, m0, dst, rt)
	struct ifnet *ifp;
	struct mbuf *m0;
	struct sockaddr *dst;
	struct rtentry *rt; /*added for osf*/
#else 
ottooutput(ifp, m0, dst)
	struct ifnet *ifp;
	struct mbuf *m0;
	struct sockaddr *dst;
#endif
{
	short type = 0;
	int s;
	int error = 0;
	union uodst {
		unsigned addr[OTTO_MAP_NADDR];
		u_char bytes[8];
	} odst;
	u_char osrc[6];
	struct ether_header *eh;
	struct mbuf *m = m0;
	int usetrailers;		/* Trailers ignored; see RFC1103 */
	struct protosw *pr;
	extern struct protosw *iffamily_to_proto();
	int len = 0;
	struct otto_softc *sc = &otto_softc[ifp->if_unit];
	unsigned vc = 0;
	unsigned gotodst = 0;
	unsigned encap;
	unsigned qq;
	struct otto_xmtq *xmtq;
	unsigned origfamily = dst->sa_family;

        if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING)) {
		error = ENETDOWN;
		goto bad;
        }

#ifdef notdef
	/* if there's stuff on the report queue, deal with it */
        if ((sc->data.reportbase[sc->data.nextreport].dmareport &
                 OTTO_REP_DMA_TAGS) != sc->data.reporttag) {
        	ottointr (ifp->if_unit);
	}
#endif

	bcopy((caddr_t)ac->ac_enaddr, (caddr_t)osrc, sizeof(osrc));
	bzero ((caddr_t)&odst, sizeof (odst));
	switch (dst->sa_family) {
#if defined(INET) || defined(__osf__)
	case AF_INET:
#ifndef  __osf__
		if (nINET == 0 || ac->ac_ipaddr.s_addr == 0) {
			cprintf ("ottooutput: %s%d: can't handle af%d\n",
				ifp->if_name, ifp->if_unit, dst->sa_family);
			error = EAFNOSUPPORT;
			goto bad;
		}
#endif 
		type = ETHERTYPE_IP;
af_inet:	{
			unsigned addr[OTTO_MAP_NADDR];
			struct in_addr idst;
			unsigned port = 0;
			unsigned protocol = 0;

			idst = ((struct sockaddr_in *)dst)->sin_addr;

			/* The following code fishes the TCP and UDP 
			   port numbers out of the packet.  This allows
			   different ports to use different virtual circuits. */
			if (sc->data.portmapping) {
				struct ip *ip;
				unsigned hlen = sizeof (struct ip);
				if (m->m_len < hlen) {
pullup:
					m = m_pullup (m, hlen);
					if (m == (struct mbuf *) 0) {
						error = ENOBUFS;
						goto bad;
					}
				}
				ip = mtod (m, struct ip *);
				protocol = ip->ip_p;
				if (protocol == IPPROTO_TCP ||
				    protocol == IPPROTO_UDP) {
					unsigned off = ntohs (ip->ip_off);
					unsigned id = ntohs (ip->ip_id);
					if ((off & ~(IP_DF|IP_MF)) == 0) {
						u_short *ps;
#ifdef __osf__
						hlen = 4*(ip->ip_vhl&0xf) + 4;
#else
						hlen = 4*ip->ip_hl + 4;
#endif
						if (m->m_len < hlen) {
							goto pullup;
						}
						ps = mtod (m, u_short *)+hlen/2;
						/* don't do ntohs here...
						   it's done in the application
						   that sets the map */
						port = ps[-2] | (ps[-1] << 16);
						if (off & IP_MF) {
							struct otto_ipc *ipc =
							  &sc->data.ipc[id %
							    OTTO_IPC_SIZ];
							ipc->id = id;
							ipc->p = protocol;
							ipc->src =
							    ip->ip_src.s_addr;
							ipc->dst =
							    ip->ip_dst.s_addr;
						        ipc->port = port;
						}
					} else {
						struct otto_ipc *ipc =
							  &sc->data.ipc[id %
							    OTTO_IPC_SIZ];
						if (ipc->id == id &&
						    ipc->p == protocol &&
						    ipc->src==ip->ip_src.s_addr &&
						    ipc->dst==ip->ip_dst.s_addr) {
							port = ipc->port;
						}
					}
					addr[0] = idst.s_addr;
					addr[1] = port;
					addr[2] = protocol;
					vc = otto_map_ip_to_vc (
						&sc->data.portvcmap, addr);
				} 
			}
			/* end of code to fish out port numbers */
			if (vc == 0) {
				addr[0] = idst.s_addr;
				addr[1] = 0;
				addr[2] = 0;
				vc = otto_map_ip_to_vc (&sc->data.ipvcmap,addr);
			}
			encap = sc->data.vcs[vc].encap;
			if (!gotodst && (vc == 0 || encap < 8)) {
				if (!arpresolve(ac, m,
#ifdef AC_IPUP
					dst,	/* OSF/1 V3.0 stupidity */
#else
					&idst,
#endif
						odst.bytes, &usetrailers)) {
					return (0);	/*if not yet resolved */
				}
			}
		}
		break;
#endif
        case AF_IMPLINK:
		ottodprintf (1, ("ottooutput: AF_IMPLINK transmit\n"));
		eh = mtod(m, struct ether_header *);		
		bcopy((caddr_t)eh->ether_dhost, (caddr_t)odst.bytes, 6);
		type = ntohs(eh->ether_type);
		m_adj(m, sizeof(struct ether_header));
		break;
#if (NPFWD > 0)
	case AF_FWD:
		ottodprintf (1, ("ottooutput: AF_FWD transmit\n"));
		eh = (struct ether_header *)dst->sa_data;
		bcopy((caddr_t)eh->ether_shost, (caddr_t)osrc,
		      sizeof(osrc));
		bcopy((caddr_t)eh->ether_dhost, (caddr_t)odst.bytes, 6);
		type = eh->ether_type;
		break;	
#endif NPFWD
	case AF_OTTO:	/* used by OTTO internally */
		{
			struct sockaddr_otto *sao =
				(struct sockaddr_otto *) dst;
			dst->sa_family = AF_UNSPEC;
			vc = sao->vc;
			encap = sc->data.vcs[vc].encap;
		}
		/* fall through */
	case AF_UNSPEC:
		ottodprintf (1, ("ottooutput: AF_UNSPEC transmit\n"));
		/* 
		 * Constructed by ARP with ethernet header. Strip dhost and
		 * type from ethernet header for use in MAC header and LLC
		 * encapsulated ethernet header. BIT ORDER FOR ALL MAC HDRS
		 * ASSUMED "CANONICAL" AS FOR ENET. (See RFC1103 and RFC1042).
		 */
		eh = (struct ether_header *)dst->sa_data;
		bcopy((caddr_t)eh->ether_dhost, (caddr_t)odst.bytes, 6);
		type = eh->ether_type;
		gotodst = 1;
		if  (type == ETHERTYPE_IP) {
			goto af_inet;	/* to get portmapping done */
		}
		break;

	default:
		ottodprintf (1, ("ottooutput: default transmit\n"));
                /*
                 * Try to find other address families and call protocol
                 * specific output routine. Protocols which perform LLC
		 * encapsulation in higher layers pass a type value of
		 * zero as an indication.
                 */
#ifndef __osf__
                if (pr = iffamily_to_proto(dst->sa_family)) {
                        (*pr->pr_ifoutput)(ifp, m, dst, &type,
                        	(char *)odst.bytes);
                } else
#endif
                {
                        limitprintf (5, ("ottooutput: %s%d: can't handle af%d\n",
				ifp->if_name, ifp->if_unit, dst->sa_family));
                        error = EAFNOSUPPORT;
                        OTTOMFREEM (m);
                        return (error);
		}
	}

	if (vc == 0) {
		vc = otto_map_to_vc (&sc->data.vcmap, odst.addr);
		if (sc->data.snmp.pkt.rvcsupport) {
			if (vc != 0) {
				unsigned flags = sc->data.vcs[vc].flags;
				if ((flags & (OTTO_VC_STALE|OTTO_VC_USED)) ==
					OTTO_VC_STALE) {
					if (otto_arp_resolve (sc, m,
						  odst.bytes, osrc, type, vc)) {
						return (0);
					}
				}
				/* can't use "flags" variable here because
				   it may be out of date if otto_arp_resolve
				   changed sc->data.vcs[vc].flags */
				sc->data.vcs[vc].flags |= OTTO_VC_USED;
			} else if ((odst.bytes[0] & 1) == 0 &&
				    otto_arp_resolve (sc, m, odst.bytes,
					osrc, type, 0)) {
				return (0);
			} else {
				/* Broadcast the packet.  This is to ensure
				   that bridges can find the destination. */
				vc = sc->data.snmp.pkt.bcastVCIout;
			}
		}
		if (vc == 0) {
			u_char *p = mtod (m, u_char *);
			static u_char arpstart[] = {
				0x00, 0x01, 0x08, 0x00, 0x06, 0x04, 0x00 };

			/* nothing in the map, try the default vc  */
			vc = sc->data.defaultvc;

			/* If that fails, try mapping target ARP addresses. */
			if (vc == 0 && m->m_len >= 28 &&
			    type == ETHERTYPE_ARP &&
			    bcmp (p, arpstart, sizeof (arpstart)) == 0 &&
			    (p[7] == 1 || p[7] == 2)) {
				union uodst ndst;
				bzero ((caddr_t) &ndst, sizeof (ndst));
				bcopy ((char *) &p[24], (char *) ndst.bytes, 4);
				if (ndst.addr[0] != 0) {
					vc = otto_map_ip_to_vc (
					    &sc->data.ipvcmap, ndst.addr);
				}
				if (vc == 0) {
					bcopy ((char *) &p[18],
						(char *) ndst.bytes, 6);
					if ((ndst.addr[0]|ndst.addr[1]) != 0) {
						vc = otto_map_to_vc (
						    &sc->data.vcmap, ndst.addr);
					}
				}
			}
			if (vc == 0) {
				error = EADDRNOTAVAIL;
				goto bad;
			}
		}
		encap = sc->data.vcs[vc].encap;
	}

	if (vc == 0 || vc >= OTTO_MAXVC) {
		printf ("otto_output: bad vc %u\n", vc);
		panic ("ottooutput: badvc");
	}

	/* add vc specific encapsulation */
	m = (*otto_encap[encap]->add) (sc, m, dst,
		osrc, odst.bytes, type, vc, &len);
	if (m == (struct mbuf *) 0) {
		ottodprintf (1, ("ottooutput: encap %d ate a pkt\n", encap));
		goto bad;
	}

	if (ottodebug >= 5) {
		int i =0;
		struct mbuf *mx = m;
		cprintf ("ottooutput: vc%d ", vc);
		while (mx != 0 && i != 96) {
			unsigned char *p = mtod (mx, unsigned char *);
			int j = 0;
			while (i != 96 && j != mx->m_len) {
				cprintf (" %02x", p[j]);
				j++;
				i++;
			}
			mx = mx->m_next;
		}
		cprintf ("\n");
	}

#ifdef __alpha
	*(unsigned *)&m->m_ipq[0] = vc;
#else
	m->m_context = vc;
#endif

	/* this is the same lock as ifp->lk_softc */
	OTTO_SMP_LOCK (&sc->data.lk_otto_softc, s);

	qq = sc->data.vcs[vc].q;

	xmtq = &sc->data.xmtqueues.per_q[qq];
	if (sc->data.xmtqueues.totallen >= sc->data.xmtqueues.maxtotallen &&
	    qq != OTTO_IFQ_SPECIAL) {
		OTTO_SMP_UNLOCK (&sc->data.lk_otto_softc, s);
		error = ENOBUFS;
		goto bad;
	}
#ifdef __osf__
        IFQ_LOCK (xmtq->pifq);
	/* this code allows AF_OTTO to get 10 extra packets into the
	   queue; other protocol are not allowed to use this reserved
	   part of the queue.
	   IP gets another special dispensation when using the normal
	   queue in order to avoid throwing away fragments after the first.
	   The IP layer checks the if_snd queue in the ifnet struct
	   to ensure it doesn't get overfull.
	   */
        if (IF_QFULL (xmtq->pifq) &&
	    (qq != OTTO_IFQ_NORMAL || origfamily != AF_INET) &&
            (origfamily != AF_OTTO ||
             xmtq->pifq->ifq_len >= xmtq->pifq->ifq_maxlen+10)) {
                IF_DROP (xmtq->pifq);
                IFQ_UNLOCK (xmtq->pifq);
		OTTO_SMP_UNLOCK (&sc->data.lk_otto_softc, s);
                error = ENOBUFS;
                goto bad;
        }
        IF_ENQUEUE_NOLOCK (xmtq->pifq, m);
        IFQ_UNLOCK (xmtq->pifq);
        NETSTAT_LOCK (&ifp->if_slock);
        microtime (&ifp->if_lastchange);
        ifp->if_obytes += len;
        if (odst.bytes[0] & 1) {
                ifp->if_omcasts++;
	}
        NETSTAT_UNLOCK(&ifp->if_slock);
#else
        if (IF_QFULL(xmtq->pifq)) {
                IF_DROP(xmtq->pifq);
		OTTO_SMP_UNLOCK (&sc->data.lk_otto_softc, s);
                error = ENOBUFS;
                goto bad;
        }
        IF_ENQUEUE (xmtq->pifq, m);
#endif
	sc->data.xmtqueues.totallen++;
	if (xmtq->next == (struct otto_xmtq *) 0 &&
	    xmtq->pktswaiting < xmtq->maxpktswaiting) {
		/* put this queue on the ready list */
		struct otto_xmtq *lastxmtq = sc->data.xmtqueues.ready;
		if (lastxmtq == (struct otto_xmtq *) 0) {
			xmtq->next = xmtq;
		} else {
			xmtq->next = lastxmtq->next;
			lastxmtq->next = xmtq;
		}
		sc->data.xmtqueues.ready = xmtq;
	}
	OTTO_IFSTART (ifp);

	OTTO_SMP_UNLOCK (&sc->data.lk_otto_softc, s);
        return(0);

bad:
	if (m != (struct mbuf *) 0) {
		OTTOMFREEM (m);
	}
	return (error);
}

/* OTTO read routine. Pass input packets up to higher levels. 
   Returns chain of mbufs that were unused, which is always the tail
   of the chain starting at m0, or m0 if none of the mbuf were used.
   The number of mbufs used is returned in *pused.

   This code assumes that the first mbuf in the chain contains the 
   link level header.  The caller must ensure that the packet is big
   enough to include this header. */
static struct mbuf *ottoread (sc, m0, bytes, vc, link, pused)
struct otto_softc *sc;
struct mbuf *m0;
unsigned bytes;
unsigned vc;
int link;
int *pused;
{
	int leftunused = bytes;
	struct mbuf *firstunused = m0;
	struct mbuf *lastused = (struct mbuf *) 0;
	struct mbuf *penultused = (struct mbuf *) 0;
	int mbufsused = 0;
	int lastusedlen;
	int oldlastusedlen;
	int len;		/* AAL5 length */
	unsigned encap;
	unsigned expectedbytes;
	unsigned savem0len;
	unsigned savem0off;

	/* Clean cache and find last mbuf containing packet. */
	while (leftunused > 0) {	/* more to clean */
		if (firstunused == (struct mbuf *) 0) {
			ottodprintf (1, ("ottoread: rcv pkt too long 1\n"));
			OTTO_STATS_INC_L (&sc->data, rcv_pkt_too_large, link);
			*pused = 0;
			return (m0);
		}
		firstunused->m_len = M_CLUSTERSZ;
		OTTO_CLEAN_DCACHE (PHYS_TO_K0 (firstunused->m_ifp),
			min (leftunused, firstunused->m_len));
		leftunused -= firstunused->m_len;
		penultused = lastused;
		lastused = firstunused;
		firstunused = firstunused->m_next;
		mbufsused++;
	}

	/* The cache is now clean.

	   firstunused now points to first unused buffer (if any).
	   mbufsused is a count of mbufs used.
	   lastused points to last used buffer, or null if mbufsused == 0.
	   penultused is the penultimate used buffer, or null if mbufsused<2
	   leftunused is -(unused bytes in last used mbuf).
	   */

	if (mbufsused == 0) {
		/* no mbufs were used */
		*pused = 0;
		return (m0);
        }

	/* The length of the data in the last used mbuf---this should 
	   end with the AAL5 trailer */
        lastusedlen = lastused->m_len + leftunused;
        ottodprintf (3, ("lastusedlen=%d\n", lastusedlen));

        if (ottodebug >= 5) {
		int i;
		unsigned char *p = mtod (m0, unsigned char *);
		for (i = 0; i != 96; i++) {
			cprintf (" %02x", p[i]);
		}
		cprintf ("\n");
        }

	/* We want to get the AAL5 length out of the AAL5 trailer. 

	   Since the trailer must come on a 48 byte boundary, and all
	   the receive mbuf sizes are multiples of four bytes, the
	   length must fall entirely in one mbuf, but it may be the
	   last or the penultimate one.
	   Even though it should never happen, I allow for the case
	   where the length is split across the two mbufs, just because
	   it's easy.
	   I drop the packet if the length is partly in the 2nd-from-
	   last mbuf.  This can happen only if the DMA fragments are
	   less than 4 bytes long, which is impossible with the current
	   hardware.  */
	if (lastusedlen >= OTTO_CRCLEN+2) {
		/* length is in last mbuf */
		unsigned char *plen = mtod (lastused, unsigned char *) +
			lastusedlen - (OTTO_CRCLEN+2);
		len = plen[1] + (plen[0] << 8);
	} else if (penultused == (struct mbuf *) 0 ||
	           penultused->m_len+lastusedlen < OTTO_CRCLEN+2) {
		/* there is no previous mbuf, or it's a very short mbuf */
		cprintf ("ottoread: rcv pkt too short 1\n");
		OTTO_STATS_INC_L (&sc->data, rcv_pkt_too_short, link);
		*pused = 0;
		return (m0);
	} else if (lastusedlen <= OTTO_CRCLEN) {
		/* Length is in penultimate mbuf */
		unsigned char *plen = mtod (penultused, unsigned char *) +
			lastusedlen + penultused->m_len - (OTTO_CRCLEN+2);
		len = plen[1] + (plen[0] << 8);
	} else {
		/* length is split across two mbufs */
		unsigned char *plen = mtod (penultused, unsigned char *) +
			penultused->m_len - 1;
		len = *mtod (lastused, unsigned char *) + (plen[0] << 8);
	}

	/* round the packet to a cell size */
	expectedbytes = (len + OTTO_MINTRAILERLEN) + OTTO_PKTROUNDUP - 1;
	expectedbytes -= (expectedbytes % OTTO_PKTROUNDUP);

	/* check that packet length field is ok */
	if (bytes < expectedbytes) {
		cprintf ("ottoread: rcv pkt too short len = %d bytes=%d expectedbytes=%d\n",
			len, bytes, expectedbytes);
		OTTO_STATS_INC_L (&sc->data, rcv_pkt_too_short, link);
		*pused = 0;
		return (m0);
	}

	/* adjust lastusedlen to remove padding and AAL5 trailer */
	lastusedlen -= bytes - len;

	/* if we aren't now using the last mbuf, adjust one back */
	if (lastusedlen <= 0) {
		if (penultused == (struct mbuf *) 0) {
			/* we're using no mbufs? should never happen */
			cprintf ("ottoread: rcv pkt has negative length?\n");
			OTTO_STATS_INC_L (&sc->data, rcv_pkt_too_short, link);
			*pused = 0;
			return (m0);
		}
		firstunused = lastused;
		lastused = penultused;
		lastusedlen += lastused->m_len;
		mbufsused--;
	}

	/* terminate chain of buffers containing packet,
	   fix up length of last mbuf */
	lastused->m_next = 0;
	oldlastusedlen = lastused->m_len;	/* save */
	lastused->m_len = lastusedlen;	/* adjust data in last used mbuf */


	savem0len = m0->m_len;
#ifndef __osf__
	savem0off = m0->m_off;
#endif

	otto_event (&sc->data, vc, bytes, OTTO_EVENT_RX);

	encap = sc->data.vcs[vc].encap;
	if ((*otto_encap[encap]->read) (sc, m0, bytes, len, vc, link)) {
		/* DECnet statistics */
		if (sc->is_ctrblk.est_bytercvd + bytes > sc->is_ctrblk.est_bytercvd) {
			sc->is_ctrblk.est_bytercvd += bytes;
		}
		if (sc->is_ctrblk.est_blokrcvd != ~0) {
			sc->is_ctrblk.est_blokrcvd++;
		}

		*pused = mbufsused;
		return (firstunused);
	}
	/* restore original chain - we used no buffers */
	lastused->m_len = oldlastusedlen;	/* restore */
	lastused->m_next = firstunused;
	/* restore position of first mbuf -- it may have been trashed */
	m0->m_len = savem0len;
#ifdef __osf__
	m0->m_data = m0->m_ext.ext_buf;
#else
	m0->m_off = savem0off;
#endif
	*pused = 0;

	return (m0);
}

/*ARGSUSED*/
static void schedtransmit (arg)
char *arg;
{
	struct sockaddr_otto st;
	struct otto_softc *sc;
	OTTO_SMP_LOCKTMP s;
	struct mbuf *m;
	struct ifnet *ifp;

	for (;;) {
		OTTO_SMP_LOCK (&otto_locksimplesend, s);
		m = otto_listsimplesend;
		if (m == (struct mbuf *) 0) {
			OTTO_SMP_UNLOCK (&otto_locksimplesend, s);
			break;
		}
		otto_listsimplesend = m->m_act;
		OTTO_SMP_UNLOCK (&otto_locksimplesend, s);
		m->m_act = (struct mbuf *) 0;

		st = *mtod (m, struct sockaddr_otto *);
		sc = st.sc;
		ifp = &sc->is_if;

		OTTO_STATS_INC (&sc->data, schedxmt);

		if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING)) {
			OTTOMFREEM (m);
		} else {
			m_adj (m, sizeof (struct sockaddr_otto));
#ifdef __osf__
			ottooutput (ifp, m, &st.dst, (struct rtentry *) 0);
#else
			ottooutput (ifp, m, &st.dst);
#endif
		}
	}
}

/* send a packet - called with softc lock held */
void otto_simplesendpkt (sc, vc, pkt, len, dst)
struct otto_softc *sc;
unsigned vc;
unsigned char *pkt;
unsigned len;
struct sockaddr *dst;
{
	struct ifnet *ifp = &sc->is_if;
	struct mbuf *m = 0;
	OTTO_SMP_LOCKTMP s;

	if (vc == 0 || vc >= OTTO_MAXVC) {
		printf ("otto_simplesendpkt: bad vc %u\n", vc);
		panic ("otto badvc");
	}
        if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING) ||
            vc >= OTTO_MAXVC) {
		return;
        }
	OTTO_STATS_INC (&sc->data, simple_send);

	if (sizeof (struct sockaddr_otto) + len > OTTOMLEN) {
		OTTOMCLGET (m);
	} else {
		OTTOMGET (m);
	}
	if (m == 0) {
		limitprintf (5, ("otto_simplesendpkt: can't get mbuf\n"));
		sc->is_ctrblk.est_sysbuf++;
	} else {
		struct sockaddr_otto *st = mtod (m, struct sockaddr_otto *);
		caddr_t p = (caddr_t) &st[1];

		st->dst = *dst;
		if (vc != 0 && st->dst.sa_family == AF_UNSPEC) {
			st->dst.sa_family = AF_OTTO;
		}
		st->vc = vc;
		st->sc = sc;
		bcopy ((char *) pkt, p, len);
		m->m_len = len + sizeof (struct sockaddr_otto);

		OTTO_SMP_LOCK (&otto_locksimplesend, s);
		m->m_act = otto_listsimplesend;
		otto_listsimplesend = m;
		OTTO_SMP_UNLOCK (&otto_locksimplesend, s);

		/* move into timer context because output routine
		   expects to be called with lock not held */
		timeout (schedtransmit, (caddr_t) 0, 0);
	}
}
#endif /* (NOTTO > 0) || defined(BINARY) */
