#if !defined(_OTTO_ARP_H_)
#define _OTTO_ARP_H_

int otto_arp_rcv (struct otto_softc *sc, caddr_t p, int len, unsigned vc,
	struct ether_header *eptr);
/* process an ARP packet in bytes p[0,...,len-1] that was received on vc "vc"
   on controller "sc".   It arrived with Ethernet header *eptr
   (the type field has been byte-swapped to host order).
   Return 1 if the packet was consumed, and 0 otherwise.
   Called with the otto softc lock held.*/

void otto_arp_sendarp19 (struct otto_softc *sc, unsigned vc);
/* Send a type 19 arp on the given vc.
   Called with the OTTO softc lock held. */

void otto_arp_sendsarp (struct otto_softc *sc, unsigned vc, unsigned tvc,
        unsigned char *uid, unsigned char *tuid, unsigned reply);
/* Send a SARP (type 21) request (reply=0) or reply (reply=1) on VC vc
   to reach uid "uid".  The target
   The targer vc and target uid are tvc and tuid.
   Called with softc lock held. */

int otto_arp_resolve (struct otto_softc *sc, struct mbuf *m,
	unsigned char *uid, unsigned char *srcuid, unsigned short type,
	unsigned vc);
/* The packet contained in "m" with ethernet type "type" is to be sent
   to UID "uid[0...5]" from UID "srcuid[0..5]" on interface "sc".
   otto_arp_resolve arranges to look up "uid" with a SARP request.
   If "vc" is non-zero, the SARP request should be sent on that vc.
   If it returns zero, the caller still owns the mbuf chain on return.
   If it returns non-zero, this call consumes the mbuf chain "m"---the
   caller should assume that it will be freed by otto_arp_resolve at
   some time in the future.
   Called with the otto softc lock not held.   Ths routine acquires and
   releases the lock internally. */

void otto_arp_tick (struct otto_softc *sc);
/* called once per second by watchdog routine.
   Sends type 9 ARPs on VCs.
   Called with softc lock held */

#endif
