#if !defined(_OTTO_ENCAPSULATE_H_)
#define _OTTO_ENCAPSULATE_H_

/* interface for encapsulation types */

#ifdef KERNEL
typedef struct mbuf *(*otto_en_add) (struct otto_softc *sc,
	struct mbuf *m,
	struct sockaddr *dst,
	unsigned char *osrc, unsigned char *odst, int type,
	unsigned vc, int *len);
/* Add encapsulation to the packet in the mbuf chain "m" appropriate for the
   destination address "dst" and the interface "sc".

   For Ethernet encapsulations, osrc is the 6 byte source address,
   odst is the 6 byte destination address, and type is Ethernet type.
   The packet is to sent on VC "vc".

   The routine should set "*len" to the total length of the packet.
   Return the resulting packet as an mbuf chain, or 0 on failure.

   The implementation usually prepends to the front of "m". */

typedef int (*otto_en_read) (struct otto_softc *sc, struct mbuf *m0,
	unsigned bytes, unsigned len, unsigned vc, int link);
/* Deliver the packet in the mbuf chain "m0" to higher levels.
   The total number of bytes DMAed into memory by the controller is "bytes".
   The AAL5 length of the packet is len.
   The packet buffers are clean in the machine's cache.
   The packet was received on vc "vc", on link "link" of OTTO interface "sc".
   Returns 1 if the packet is consumed and 0 otherwise. */

struct otto_en {
	otto_en_add add;		/* routine to add encapsulation */
	otto_en_read read;		/* routine to handle received packets */
};

extern struct otto_en *otto_encap[];	/* encapsulations: indexed by
						numbers below */
#endif

/* General purpose encapsulations are in the range 0..7.
   These encapsulations put UIDs in the packet and so can be bridged. */
#define OTTO_EN_ATMF	0	/* ATM Forum ROW encapsulation */
#define OTTO_EN_SLE	1	/* Synoptics LAN emulation
				   just like ATMF but without LLC header */
#define OTTO_EN_FDDI	2	/* Gigaswitch FDDI encasulation */

/* Encapsulations in the range 8..15 have no UIDs in the packet,
   so can't be bridged. */
#define OTTO_EN_RAW	8	/* No header at all. */
#define OTTO_EN_LLC	9	/* LLC header only  (probably
				   "Classic IP" braindamage) */
#define OTTO_EN_CRED    0xc     /* credit VC encapsulation */
#endif
