#ifndef _OTTO_MEASURE_H_
#define _OTTO_MEASURE_H_ 1

/* arrival/departure time measurement states */
#define OTTO_TIMES_NULL 0       /* no memory allocated */
#define OTTO_TIMES_HOLD 1       /* measurements waiting for read */
#define OTTO_TIMES_COLLECT 2    /* in process of collecting measurements

/* event types */
#define OTTO_EVENT_RX 0		/* receive event */
#define OTTO_EVENT_TX 1		/* transmit event */
#define OTTO_EVENT_TICK 2	/* timekeeping event */

/* packet/cell count measurement structures */
struct otto_counts {
	struct otto_perevent_counts {
		unsigned pkts;	/* packet count */
		unsigned cells;	/* cell count */
	} perevent[2];		/* for either OTTO_EVENT_RX or OTTO_EVENT_TX */
};


#define OTTO_EVENT_MAKE(_type,_vc)	(((_vc) << 4) | (_type))

#define OTTO_EVENTS_MAX	10000	/* Memimum events allowed */

/* An event */
struct otto_event {
	unsigned cycles;		/* cycle counter reading at event */
	unsigned short len;		/* packet length */
	unsigned short vctype;		/* event type and VC */
#define OTTO_EVENT_TYPE(_vctype)  ((_vctype) & 0xf)	/* return event type */
#define OTTO_EVENT_VC(_vctype)	((_vctype) >> 4)	/* return event VC */
};

/* arrival/departure time measurement structures */
struct otto_times {
	unsigned vclo;		/* data collected for vclo <= vc < vchi */
	unsigned vchi;
	struct timeval starttime;	/* time collection started */
	struct timeval endtime;		/* time collection ended */
	unsigned startcycles;		/* cycle counter at starttime */
	unsigned endcycles;		/* cycle counter at endtime */
	struct otto_event *events;	/* collection table */
	unsigned maxevents;		/* max number of events samples */
	unsigned nevents;		/* current number of events */
	unsigned state;			/* collection state machine */
};

#ifdef KERNEL
/* measurement collection procedure */
struct otto_data;
void otto_event (struct otto_data *od, unsigned vc,
	unsigned size, unsigned eventtype);
#endif

/* measurement ioctl data structures */
struct otto_measure {
	unsigned cmd;				/* measurement command */
	unsigned vc;				/* VCI, unused for totals */
	union {
		struct otto_counts counts; 	/* counters */
		struct otto_times times; 	/* time measurements */
	} uu;
};

/* measurement ioctl definitions */
#define OTTO_MEASURE		_IOWR('o', 32, struct otto_measure)

/* measurement command definitions */

#define OTTO_COUNTERS 1
#define OTTO_TIMERS 2
#define OTTO_CT(_x) (((_x) >> 24) & 0xff)

/* counter actions */
#define OTTO_C_ZERO 0
#define OTTO_C_GET 1

#define OTTO_C_MK(_a)  ((OTTO_COUNTERS << 24) | (_a))
#define OTTO_C_ACT(_x) ((_x) & 7)

/* timer actions */
#define OTTO_T_START 0
#define OTTO_T_STOP 1
#define OTTO_T_GET 2
#define OTTO_T_CLEAR 3

#define OTTO_T_MK(_a)  ((OTTO_TIMERS<<24) | (_a))
#define OTTO_T_ACT(_x) ((_x) & 7)
#endif /* _OTTO_MEASURE_H_ */
