/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright (c) Digital Equipment Corporation, 1991, 1994    *
 * *                                                               *
 * *   All Rights Reserved.  Unpublished rights  reserved  under   *
 * *   the copyright laws of the United States.                    *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Digital   *
 * *   Equipment Corporation.  Possession, use,  duplication  or   *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Digital Equipment  *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
#include <stdio.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <netdb.h>
#include "otto_ioctl.h"
#include "otto_map.h"

static int	 fd = -1;
static char	 *progname;

static void open_device(char *devname)
{
  char filename[40];

  strcpy(filename, "/dev/");
  strcat(filename, devname);
  if ((fd = open(filename, O_RDONLY, 0)) < 0) {
    perror("open");
    exit(1);
  }
}

static void close_device()
{
  (void) close(fd);
  fd = -1;
}

void usage()
{
  fprintf(stderr,
	  "usage: %s device-name -p sonet | sdh\n",
	  progname);
  if (fd != -1)
    close_device();
  exit(1);
}

int main(int argc, char **argv)
{
  int             ac  = argc;
  char          **av  = argv;
  int             mod = 0;
  unsigned        sdh;

  progname = *(av++);
  ac--;
  
  if (ac == 0)
    usage();

  open_device(*(av++));
  ac--;
  
  while (ac)
    {
      if (**av == '-')
	switch ((*av)[1])
	  {
	  case 'p' :
	    av++;
	    if (--ac)
	      {
		if      (strcmp(*av, "sdh")   == 0)
		  {
		    sdh = 1;
		    mod++;
		  }
		else if (strcmp(*av, "sonet") == 0)
		  {
		    sdh = 0;
		    mod++;
		  }
		else
		  {
		    fprintf(stderr, "%s - Unknown Phy Type '%s'\n",
			    progname, *av);
		    usage();
		  }
	      }
	    else
	      {
		fprintf(stderr, "%s - Missing PHY Value\n", progname);
		usage();
	      }
	    break;
	  default:
	    fprintf(stderr, "%s - Unknown Switch '-%c'\n",
		    progname, (*av)[1]);
	    usage();
	  }
      else
	usage();
      av++;
      ac--;
    }
  
  if (mod)
    if (ioctl(fd, OTTO_SET_FRAMING, (char *) &sdh) == -1)
      perror("ioctl");
  printf("%s Configuration:\n", argv[1]);
  printf("    Phy = %s\n", sdh ? "SDH" : "Sonet");
  
  close_device();
  
  return (0);
}

